@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -54,7 +53,7 @@ my_option *Abstract_program::get_options_array() { return &this->m_options[0]; }
 
 Abstract_program::Abstract_program()
     : m_debug_options(this), m_help_options(this) {
-  this->add_providers(&this->m_help_options, &this->m_debug_options, NULL);
+  this->add_providers(&this->m_help_options, &this->m_debug_options, nullptr);
 }
 
 void Abstract_program::run(int argc, char **argv) {
@@ -70,8 +69,8 @@ void Abstract_program::run(int argc, char **argv) {
                              Message_type_error));
   my_getopt_use_args_separator = false;
 
-  int ho_error = handle_options(&argc, &argv, this->get_options_array(),
-                                Abstract_program::callback_option_parsed);
+  const int ho_error = handle_options(&argc, &argv, this->get_options_array(),
+                                      Abstract_program::callback_option_parsed);
   if (ho_error != 0) {
     this->error(Message_data(ho_error, "Error during handling options",
                              Message_type_error));
@@ -98,7 +97,7 @@ void Abstract_program::init_name(char *name_from_cmd_line) {
   char *name;
 
   char name_buf[FN_REFLEN];
-  if (GetModuleFileName(NULL, name_buf, FN_REFLEN) != 0) {
+  if (GetModuleFileName(nullptr, name_buf, FN_REFLEN) != 0) {
     name = name_buf;
   } else {
     name = name_from_cmd_line;
