@@ -22,17 +22,25 @@
 
 /*
   This is a test executable which verifies that Json_wrapper::seek,
-  Json_dom::seek and Json_dom::parse functions are visible and can be called.
+  Json_dom::seek, Json_dom::parse, json_binary::parse_binary,
+  json_binary::serialize functions are visible and can be called.
  */
 #include <cstring>
 #include <iostream>
 #include <string>
 
+#include "sql-common/json_binary.h"
 #include "sql-common/json_dom.h"
 #include "sql-common/json_path.h"
 
+#include "sql-common/json_error_handler.h"
 #include "sql_string.h"
 
+void CoutDefaultDepthHandler() { std::cout << "Doc too deep"; }
+void CoutDefaultKeyErrorHandler() { std::cout << "Key too big"; }
+void CoutDefaultValueErrorHandler() { std::cout << "Value too big"; }
+void InvalidJsonErrorHandler() { std::cout << "Invalid JSON"; }
+
 int main() {
   Json_object o;
   for (size_t i = 0; i < 1000; ++i) {
@@ -52,6 +60,19 @@ int main() {
 
   Json_wrapper wr(&o);
   wr.set_alias();
+
+  StringBuffer<20000> buf;
+  if (wr.to_binary(&buf, CoutDefaultDepthHandler, CoutDefaultKeyErrorHandler,
+                   CoutDefaultValueErrorHandler, InvalidJsonErrorHandler)) {
+    std::cout << "error";
+  } else {
+    const json_binary::Value v(
+        json_binary::parse_binary(buf.ptr(), buf.length()));
+    std::string std_string;
+    v.to_pretty_std_string(&std_string, CoutDefaultDepthHandler);
+    std::cout << "1. Binary_val[" << std_string << "]" << std::endl;
+  }
+
   wr.seek(path, path.leg_count(), &hits, true, need_only_one);
 
   for (auto &hit : hits) {
@@ -97,5 +118,43 @@ int main() {
     if (dom != nullptr) assert(false);
   }
 
+  {
+    my_decimal m;
+    double2my_decimal(0, 3.14000000001, &m);
+    const Json_decimal jd(m);
+    if (json_binary::serialize(&jd, &buf, CoutDefaultDepthHandler,
+                               CoutDefaultKeyErrorHandler,
+                               CoutDefaultValueErrorHandler))
+      std::cout << "ERRROR!!" << std::endl;
+
+    json_binary::Value v = json_binary::parse_binary(buf.ptr(), buf.length());
+    std::string std_string;
+    if (v.to_std_string(&std_string, CoutDefaultDepthHandler))
+      std::cout << "ERRROR!!" << std::endl;
+    std::cout << "4. val[" << std_string << "]" << std::endl;
+  }
+
+  {
+    /* DATETIME scalar */
+    MYSQL_TIME dt;
+    std::memset(&dt, 0, sizeof dt);
+    dt.year = 1988;
+    dt.month = 12;
+    dt.day = 15;
+    dt.time_type = MYSQL_TIMESTAMP_DATE;
+    const Json_datetime jd(dt, MYSQL_TYPE_DATETIME);
+
+    if (json_binary::serialize(&jd, &buf, CoutDefaultDepthHandler,
+                               CoutDefaultKeyErrorHandler,
+                               CoutDefaultValueErrorHandler))
+      std::cout << "ERRROR!!" << std::endl;
+
+    json_binary::Value v = json_binary::parse_binary(buf.ptr(), buf.length());
+    std::string std_string;
+    if (v.to_pretty_std_string(&std_string, CoutDefaultDepthHandler))
+      std::cout << "ERRROR!!" << std::endl;
+    std::cout << "5. val[" << std_string << "]" << std::endl;
+  }
+
   return 0;
 }
