@@ -1,16 +1,15 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,6 +28,7 @@
 #include "plugin/group_replication/include/consistency_manager.h"
 #include "plugin/group_replication/include/handlers/pipeline_handlers.h"
 #include "plugin/group_replication/include/plugin.h"
+#include "string_with_len.h"
 
 using std::string;
 
@@ -109,13 +109,13 @@ int Certification_handler::handle_event(Pipeline_event *pevent,
                                         Continuation *cont) {
   DBUG_TRACE;
 
-  Log_event_type ev_type = pevent->get_event_type();
+  mysql::binlog::event::Log_event_type ev_type = pevent->get_event_type();
   switch (ev_type) {
-    case binary_log::TRANSACTION_CONTEXT_EVENT:
+    case mysql::binlog::event::TRANSACTION_CONTEXT_EVENT:
       return handle_transaction_context(pevent, cont);
-    case binary_log::GTID_LOG_EVENT:
+    case mysql::binlog::event::GTID_LOG_EVENT:
       return handle_transaction_id(pevent, cont);
-    case binary_log::VIEW_CHANGE_EVENT:
+    case mysql::binlog::event::VIEW_CHANGE_EVENT:
       return extract_certification_info(pevent, cont);
     default:
       next(pevent, cont);
