@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,12 +36,15 @@
 #include <mysqld_error.h>
 #include <sys/types.h>
 #include "plugin/replication_observers_example/gr_message_service_example.h"
+#include "plugin/replication_observers_example/src/binlog/service/iterator/tests/pfs.h"
+#include "plugin/replication_observers_example/src/binlog/service/iterator/tests/status_vars.h"
 
 #include <include/mysql/components/services/ongoing_transaction_query_service.h>
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "sql/current_thd.h"
 #include "sql/sql_class.h"
+#include "string_with_len.h"
 
 static MYSQL_PLUGIN plugin_info_ptr;
 static SERVICE_TYPE(registry) *reg_srv = nullptr;
@@ -200,8 +203,7 @@ static void dump_transaction_calls() {
 /*
   Transaction lifecycle events observers.
 */
-static int trans_before_dml(Trans_param *,
-                            int &out_val MY_ATTRIBUTE((unused))) {
+static int trans_before_dml(Trans_param *, int &out_val [[maybe_unused]]) {
   trans_before_dml_call++;
 
   DBUG_EXECUTE_IF("cause_failure_in_before_dml_hook", out_val = 1;);
@@ -253,7 +255,7 @@ static int before_commit_tests(Trans_param *param,
 
     case POSITIVE_CERTIFICATION_WITH_GTID:
       fake_sid.parse("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
-                     binary_log::Uuid::TEXT_LENGTH);
+                     mysql::gtid::Uuid::TEXT_LENGTH);
       fake_sidno = get_sidno_from_global_sid_map(fake_sid);
       fake_gno = get_last_executed_gno(fake_sidno);
       fake_gno++;
@@ -293,7 +295,7 @@ static int before_commit_tests(Trans_param *param,
 }
 #endif
 
-static int trans_before_commit(Trans_param *param MY_ATTRIBUTE((unused))) {
+static int trans_before_commit(Trans_param *param [[maybe_unused]]) {
   trans_before_commit_call++;
 
   DBUG_EXECUTE_IF("force_error_on_before_commit_listener", return 1;);
@@ -323,7 +325,7 @@ static int trans_before_rollback(Trans_param *) {
 }
 
 static int trans_after_commit(Trans_param *) {
-  DBUG_EXECUTE_IF("group_replication_before_commit_hook_wait", {
+  DBUG_EXECUTE_IF("bgc_after_after_commit_stage", {
     const char act[] = "now wait_for continue_commit";
     assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
   });
@@ -333,7 +335,7 @@ static int trans_after_commit(Trans_param *) {
   return 0;
 }
 
-static int trans_after_rollback(Trans_param *param MY_ATTRIBUTE((unused))) {
+static int trans_after_rollback(Trans_param *param [[maybe_unused]]) {
   trans_after_rollback_call++;
 
   DBUG_EXECUTE_IF("validate_replication_observers_plugin_server_requirements",
@@ -342,8 +344,8 @@ static int trans_after_rollback(Trans_param *param MY_ATTRIBUTE((unused))) {
   return 0;
 }
 
-static int trans_begin(Trans_param *param MY_ATTRIBUTE((unused)),
-                       int &out_val MY_ATTRIBUTE((unused))) {
+static int trans_begin(Trans_param *param [[maybe_unused]],
+                       int &out_val [[maybe_unused]]) {
   trans_begin_call++;
 
   return 0;
@@ -500,12 +502,12 @@ int validate_plugin_server_requirements(Trans_param *param) {
   */
   rpl_sid fake_sid;
   fake_sid.parse("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
-                 binary_log::Uuid::TEXT_LENGTH);
-  rpl_sidno fake_sidno = get_sidno_from_global_sid_map(fake_sid);
-  rpl_gno fake_gno = get_last_executed_gno(fake_sidno) + 1;
+                 mysql::gtid::Uuid::TEXT_LENGTH);
+  const rpl_sidno fake_sidno = get_sidno_from_global_sid_map(fake_sid);
+  const rpl_gno fake_gno = get_last_executed_gno(fake_sidno) + 1;
 
-  Gtid gtid = {fake_sidno, fake_gno};
-  Gtid_specification gtid_spec = {ASSIGNED_GTID, gtid};
+  const Gtid gtid = {fake_sidno, fake_gno};
+  const Gtid_specification gtid_spec = {ASSIGNED_GTID, gtid};
   Gtid_log_event *gle =
       new Gtid_log_event(param->server_id, true, 0, 1, true, 0, 0, gtid_spec,
                          UNKNOWN_SERVER_VERSION, UNKNOWN_SERVER_VERSION);
@@ -522,7 +524,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
   /*
     Instantiate a anonymous Gtid_log_event without a THD parameter.
   */
-  Gtid_specification anonymous_gtid_spec = {ANONYMOUS_GTID, gtid};
+  const Gtid_specification anonymous_gtid_spec = {ANONYMOUS_GTID, gtid};
   gle = new Gtid_log_event(param->server_id, true, 0, 1, true, 0, 0,
                            anonymous_gtid_spec, UNKNOWN_SERVER_VERSION,
                            UNKNOWN_SERVER_VERSION);
@@ -544,7 +546,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
 
   if (tcle->is_valid()) {
     Gtid_set *snapshot_version = tcle->get_snapshot_version();
-    size_t snapshot_version_len = snapshot_version->get_encoded_length();
+    const size_t snapshot_version_len = snapshot_version->get_encoded_length();
     uchar *snapshot_version_buf =
         (uchar *)my_malloc(PSI_NOT_INSTRUMENTED, snapshot_version_len, MYF(0));
     snapshot_version->encode(snapshot_version_buf);
@@ -589,7 +591,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
   get_server_startup_prerequirements(startup_pre_reqs);
 
   // check the server is initialized by checking if the default channel exists
-  bool server_engine_ready = channel_is_active("", CHANNEL_NO_THD);
+  const bool server_engine_ready = channel_is_active("", CHANNEL_NO_THD);
 
   uchar *encoded_gtid_executed = nullptr;
   size_t length;
@@ -632,7 +634,7 @@ int validate_plugin_server_requirements(Trans_param *param) {
 }
 
 int test_channel_service_interface_initialization() {
-  int error = initialize_channel_service_interface();
+  const int error = initialize_channel_service_interface();
   assert(error);
   return error;
 }
@@ -682,8 +684,8 @@ int test_channel_service_interface() {
   // Get the last delivered gno (should be 0)
   rpl_sid fake_sid;
   fake_sid.parse("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
-                 binary_log::Uuid::TEXT_LENGTH);
-  rpl_sidno fake_sidno = get_sidno_from_global_sid_map(fake_sid);
+                 mysql::gtid::Uuid::TEXT_LENGTH);
+  const rpl_sidno fake_sidno = get_sidno_from_global_sid_map(fake_sid);
   rpl_gno gno = channel_get_last_delivered_gno(interface_channel, fake_sidno);
   assert(gno == 0);
 
@@ -743,8 +745,8 @@ int test_channel_service_interface() {
 
   // Extract the applier ids
   applier_id = nullptr;
-  int num_appliers = channel_get_thread_id(interface_channel,
-                                           CHANNEL_APPLIER_THREAD, &applier_id);
+  const int num_appliers = channel_get_thread_id(
+      interface_channel, CHANNEL_APPLIER_THREAD, &applier_id);
   assert(num_appliers == 4);
 
   unsigned long thread_id = 0;
@@ -797,7 +799,7 @@ int test_channel_service_interface_io_thread() {
   char interface_channel[] = "example_channel";
 
   // Assert the channel exists
-  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
   assert(exists);
 
   // Assert that the receiver is running
@@ -806,8 +808,8 @@ int test_channel_service_interface_io_thread() {
 
   // Extract the receiver id
   long unsigned int *thread_id = nullptr;
-  int num_threads = channel_get_thread_id(interface_channel,
-                                          CHANNEL_RECEIVER_THREAD, &thread_id);
+  const int num_threads = channel_get_thread_id(
+      interface_channel, CHANNEL_RECEIVER_THREAD, &thread_id);
   assert(num_threads == 1);
   assert(*thread_id > 0);
   my_free(thread_id);
@@ -821,7 +823,7 @@ int test_channel_service_interface_io_thread() {
   my_free(retrieved_gtid_set);
 
   // Check that the applier thread is waiting for events to be queued.
-  int is_waiting = channel_is_applier_waiting(interface_channel);
+  const int is_waiting = channel_is_applier_waiting(interface_channel);
   assert(is_waiting == 1);
 
   // Stop the channel
@@ -837,7 +839,7 @@ int test_channel_service_interface_io_thread() {
 
 bool test_channel_service_interface_is_io_stopping() {
   // The initialization method should return OK
-  int error = initialize_channel_service_interface();
+  bool error = initialize_channel_service_interface();
   assert(!error);
 
   // Initialize the channel to be used with the channel service interface
@@ -863,7 +865,7 @@ bool test_channel_service_interface_is_io_stopping() {
   assert(!error);
 
   // Assert the channel exists
-  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
   assert(exists);
 
   // Wait until I/O thread reached the error and is going to stop
@@ -880,12 +882,12 @@ bool test_channel_service_interface_is_io_stopping() {
   assert(!error);
 
   // Assert that the receiver is stopping
-  bool io_stopping =
+  const bool io_stopping =
       channel_is_stopping(interface_channel, CHANNEL_RECEIVER_THREAD);
   assert(io_stopping);
 
   // Assert that the receiver is running
-  bool io_running =
+  const bool io_running =
       channel_is_active(interface_channel, CHANNEL_RECEIVER_THREAD);
   assert(io_running);
 
@@ -910,7 +912,7 @@ bool test_channel_service_interface_is_io_stopping() {
 
 bool test_channel_service_interface_is_sql_stopping() {
   // The initialization method should return OK
-  int error = initialize_channel_service_interface();
+  bool error = initialize_channel_service_interface();
   assert(!error);
 
   // Initialize the channel to be used with the channel service interface
@@ -921,7 +923,7 @@ bool test_channel_service_interface_is_sql_stopping() {
   assert(!error);
 
   // Assert the channel exists
-  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
   assert(exists);
 
   // Unregister the thread stop hook
@@ -955,12 +957,12 @@ bool test_channel_service_interface_is_sql_stopping() {
   assert(!error);
 
   // Assert that the applier is stopping
-  bool sql_stopping =
+  const bool sql_stopping =
       channel_is_stopping(interface_channel, CHANNEL_APPLIER_THREAD);
   assert(sql_stopping);
 
   // Assert that the applier is running
-  bool sql_running =
+  const bool sql_running =
       channel_is_active(interface_channel, CHANNEL_APPLIER_THREAD);
   assert(sql_running);
 
@@ -985,7 +987,7 @@ bool test_channel_service_interface_is_sql_stopping() {
 
 bool test_channel_service_interface_relay_log_renamed() {
   // The initialization method should return OK
-  int error = initialize_channel_service_interface();
+  bool error = initialize_channel_service_interface();
   assert(!error);
 
   // Initialize the channel to be used with the channel service interface
@@ -1001,7 +1003,7 @@ bool test_channel_service_interface_relay_log_renamed() {
   assert(!error);
 
   // Assert the channel exists
-  bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
+  const bool exists = channel_is_active(interface_channel, CHANNEL_NO_THD);
   assert(exists);
 
   // Start the SQL thread
@@ -1034,11 +1036,10 @@ bool test_server_count_transactions() {
 
   assert(service.is_valid());
 
-  unsigned long *ids = NULL;
+  unsigned long *ids = nullptr;
   unsigned long size = 0;
   bool error = service->get_ongoing_server_transactions(&ids, &size);
   assert(!error);
-  fprintf(stderr, "[DEBUG:] Counting transactions! %lu \n", size);
 
   assert(size == 3);
 
@@ -1101,6 +1102,22 @@ static int replication_observers_example_plugin_init(MYSQL_PLUGIN plugin_info) {
     return 1;
   }
 
+  if (binlog::service::iterators::tests::register_pfs_tables()) {
+    /* purecov: begin inspected */
+    LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Failure on init PFS tables");
+    deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
+    return 1;
+    /* purecov: end */
+  }
+
+  if (binlog::service::iterators::tests::register_status_variables()) {
+    /* purecov: begin inspected */
+    LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "Failure on init STATS VARS");
+    deinit_logging_service_for_plugin(&reg_srv, &log_bi, &log_bs);
+    return 1;
+    /* purecov: end */
+  }
+
   LogPluginErr(INFORMATION_LEVEL, ER_LOG_PRINTF_MSG,
                "replication_observers_example_plugin: init finished");
 
@@ -1130,6 +1147,9 @@ static int replication_observers_example_plugin_deinit(void *p) {
   dump_transaction_calls();
   dump_binlog_relay_calls();
 
+  binlog::service::iterators::tests::unregister_status_variables();
+  binlog::service::iterators::tests::unregister_pfs_tables();
+
   if (unregister_server_state_observer(&server_state_observer, p)) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG,
                  "Failure in unregistering the server state observers");
