@@ -1,17 +1,16 @@
 /*
-  Copyright (c) 2022, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2022, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,18 +26,18 @@
 
 #include <chrono>
 #include <memory>
-#include <system_error>
 
 #include "basic_protocol_splicer.h"
 #include "classic_connection_base.h"
 #include "classic_frame.h"
-#include "destination_error.h"
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/net_ts/impl/poll.h"
 #include "mysql/harness/net_ts/internet.h"
 #include "mysql/harness/stdx/expected.h"
 #include "mysql/harness/utility/string.h"  // join
 #include "mysqlrouter/connection_pool_component.h"
+#include "mysqlrouter/datatypes.h"
+#include "mysqlrouter/routing.h"
 #include "mysqlrouter/routing_component.h"
 #include "mysqlrouter/utils.h"  // to_string
 #include "processor.h"
@@ -130,6 +129,20 @@ static stdx::expected<std::error_code, std::error_code> sock_error_code(
   return {};
 }
 
+// skip destinations which don't matched the current expected server-mode.
+static bool skip_destination(MysqlRoutingClassicConnectionBase *conn,
+                             Destination *destination) {
+  if (conn->context().access_mode() != routing::AccessMode::kAuto) return false;
+
+  const auto conn_server_mode = conn->expected_server_mode();
+  const auto dest_server_mode = destination->server_mode();
+
+  return ((conn_server_mode == mysqlrouter::ServerMode::ReadOnly &&
+           dest_server_mode == mysqlrouter::ServerMode::ReadWrite) ||
+          (conn_server_mode == mysqlrouter::ServerMode::ReadWrite &&
+           dest_server_mode == mysqlrouter::ServerMode::ReadOnly));
+}
+
 stdx::expected<Processor::Result, std::error_code>
 ConnectProcessor::init_destination() {
   std::vector<std::string> dests;
@@ -143,40 +156,66 @@ ConnectProcessor::init_destination() {
                                    mysql_harness::join(dests, ",")));
   }
 
-  // reset the error-code for this destination.
-  destination_ec_.clear();
+  trace_event_connect_ =
+      trace_span(parent_event_, "mysql/from_pool_or_connect");
+  if (auto *ev = trace_event_connect_) {
+    ev->attrs.emplace_back("mysql.remote.candidates",
+                           mysql_harness::join(dests, ","));
+  }
 
-  all_quarantined_ = true;
+  // adjust the expected-server-mode depending if we have:
+  //
+  // - RW, RO
+  // - only RW (multi-primary)
+  // - only RO (replica of replicaset)
+  if (connection()->context().access_mode() == routing::AccessMode::kAuto) {
+    bool has_read_only{false};
+    bool has_read_write{false};
+
+    for (auto const &dest : destinations_) {
+      if (dest->server_mode() == mysqlrouter::ServerMode::ReadOnly) {
+        has_read_only = true;
+      }
+      if (dest->server_mode() == mysqlrouter::ServerMode::ReadWrite) {
+        has_read_write = true;
+      }
+    }
+
+    if (has_read_only && !has_read_write) {
+      connection()->expected_server_mode(mysqlrouter::ServerMode::ReadOnly);
+    } else if (!has_read_only && has_read_write) {
+      connection()->expected_server_mode(mysqlrouter::ServerMode::ReadWrite);
+    }
+  }
 
   destinations_it_ = destinations_.begin();
-  if (destinations_it_ == destinations_.end()) {
-    if (connect_errors_.empty()) {
+
+  if (destinations_it_ != destinations_.end()) {
+    const auto &destination = *destinations_it_;
+
+    if (connection()->context().access_mode() == routing::AccessMode::kAuto) {
+      if (skip_destination(connection(), destination.get())) {
+        stage(Stage::NextDestination);
+
+        return Result::Again;
+      }
+    }
+
+    stage(is_destination_good(destination->hostname(), destination->port())
+              ? Stage::Resolve
+              : Stage::NextDestination);
+
+    return Result::Again;
+  } else {
+    if (!last_ec_) {
       // no backends
       log_debug("init_destination(): the destinations list is empty");
-
-      connect_errors_.emplace_back(
-          "no destinations",
-          make_error_code(DestinationsErrc::kNoDestinations));
+      last_ec_ = make_error_code(DestinationsErrc::kNoDestinations);
     }
 
     stage(Stage::Error);
     return Result::Again;
   }
-
-  const auto &destination = *destinations_it_;
-
-  if (is_destination_good(destination->hostname(), destination->port())) {
-    stage(Stage::Resolve);
-  } else {
-    connect_errors_.emplace_back(
-        "connect(/* " + destination->hostname() + ":" +
-            std::to_string(destination->port()) + " */)",
-        make_error_code(DestinationsErrc::kQuarantined));
-
-    stage(Stage::NextDestination);
-  }
-
-  return Result::Again;
 }
 
 stdx::expected<Processor::Result, std::error_code> ConnectProcessor::resolve() {
@@ -208,49 +247,20 @@ stdx::expected<Processor::Result, std::error_code> ConnectProcessor::resolve() {
     }
   }
 
-  auto started = std::chrono::steady_clock::now();
-
   const auto resolve_res = resolver_.resolve(
       destination->hostname(), std::to_string(destination->port()));
 
   if (!resolve_res) {
-    auto ec = resolve_res.error();
-
-    const auto resolve_duration =
-        std::chrono::duration_cast<std::chrono::milliseconds>(
-            std::chrono::steady_clock::now() - started);
-    connect_errors_.emplace_back(
-        "resolve(" + destination->hostname() + ") failed after " +
-            std::to_string(resolve_duration.count()) + "ms",
-        ec);
-
-    log_debug("resolve(%s,%d) failed: %s:%s",  //
-              destination->hostname().c_str(), destination->port(),
-              ec.category().name(), ec.message().c_str());
-
-    destination_ec_ = ec;
-
-    // resolve(...) failed, move host:port to the quarantine to monitor the
-    // solve to come back.
-
-    auto hostname = destination->hostname();
-    auto port = destination->port();
-
-    auto &ctx = connection()->context();
-
-    if (ctx.shared_quarantine().update({hostname, port}, false)) {
-      log_debug("[%s] add destination '%s:%d' to quarantine",
-                ctx.get_name().c_str(), hostname.c_str(), port);
-    } else {
-      // failed to connect, but not quarantined. Don't close the ports, yet.
-      all_quarantined_ = false;
-    }
+    log_debug("resolve(%s,%d) failed: %s:%s", destination->hostname().c_str(),
+              destination->port(), resolve_res.error().category().name(),
+              resolve_res.error().message().c_str());
+    destination->connect_status(resolve_res.error());
 
     stage(Stage::NextDestination);
     return Result::Again;
   }
 
-  endpoints_ = *resolve_res;
+  endpoints_ = resolve_res.value();
 
 #if 0
   std::cerr << __LINE__ << ": " << destination->hostname() << "\n";
@@ -277,7 +287,11 @@ stdx::expected<Processor::Result, std::error_code>
 ConnectProcessor::init_connect() {
   // trace(Tracer::Event().stage("connect::init_connect"));
 
-  (void)connection()->socket_splicer()->server_conn().close();
+  auto tcp_conn = dynamic_cast<TcpConnection *>(
+      connection()->socket_splicer()->server_conn().connection().get());
+
+  // close socket if it is already open
+  if (tcp_conn) (void)tcp_conn->close();
 
   connection()->connect_error_code({});  // reset the connect-error-code.
 
@@ -292,7 +306,7 @@ ConnectProcessor::init_connect() {
 stdx::expected<Processor::Result, std::error_code>
 ConnectProcessor::from_pool() {
   auto *socket_splicer = connection()->socket_splicer();
-  auto client_protocol = connection()->client_protocol();
+  auto *client_protocol = connection()->client_protocol();
 
   if (!client_protocol->client_greeting()) {
     // taking a connection from the pool requires that the client's greeting
@@ -301,6 +315,9 @@ ConnectProcessor::from_pool() {
     return Result::Again;
   }
 
+  trace_event_socket_from_pool_ =
+      trace_span(trace_event_connect_, "mysql/from_pool");
+
   auto &pools = ConnectionPoolComponent::get_instance();
 
   if (auto pool = pools.get(ConnectionPoolComponent::default_pool_name())) {
@@ -314,8 +331,12 @@ ConnectProcessor::from_pool() {
     client_caps
         // connection specific.
         .reset(classic_protocol::capabilities::pos::ssl)
+        .reset(classic_protocol::capabilities::pos::query_attributes)
         .reset(classic_protocol::capabilities::pos::compress)
         .reset(classic_protocol::capabilities::pos::compress_zstd)
+        .reset(classic_protocol::capabilities::pos::session_track)
+        .reset(classic_protocol::capabilities::pos::
+                   text_result_with_session_tracking)
         // session specific capabilities which can be recovered by
         // set_server_option()
         .reset(classic_protocol::capabilities::pos::multi_statements);
@@ -326,8 +347,12 @@ ConnectProcessor::from_pool() {
           auto pooled_caps = pooled_conn.shared_capabilities();
 
           pooled_caps.reset(classic_protocol::capabilities::pos::ssl)
+              .reset(classic_protocol::capabilities::pos::query_attributes)
               .reset(classic_protocol::capabilities::pos::compress)
               .reset(classic_protocol::capabilities::pos::compress_zstd)
+              .reset(classic_protocol::capabilities::pos::session_track)
+              .reset(classic_protocol::capabilities::pos::
+                         text_result_with_session_tracking)
               .reset(classic_protocol::capabilities::pos::multi_statements);
 
           return (pooled_conn.endpoint() == ep &&  //
@@ -358,6 +383,34 @@ ConnectProcessor::from_pool() {
         (void)socket_splicer->server_conn().connection()->set_io_context(
             socket_splicer->client_conn().connection()->io_ctx());
 
+        // reset the seq-id of the server side as this is a new command.
+        if (connection()->server_protocol() != nullptr) {
+          connection()->server_protocol()->seq_id(0xff);
+        }
+
+        if (connection()->expected_server_mode() ==
+            mysqlrouter::ServerMode::Unavailable) {
+          const auto *dest = destinations_it_->get();
+          // before the first query, the server-mode is not set, remember it
+          // now.
+          connection()->expected_server_mode(dest->server_mode());
+        }
+
+        // set destination-id to get the "trace_set_connection_attributes"
+        // right.
+        connection()->destination_id(
+            destination_id_from_endpoint(*endpoints_it_));
+
+        if (auto *ev = trace_event_socket_from_pool_) {
+          trace_set_connection_attributes(ev);
+          trace_span_end(ev);
+        }
+
+        // update the msg-tracer callback to the new connection.
+        if (auto *server_ssl = socket_splicer->server_channel()->ssl()) {
+          SSL_set_msg_callback_arg(server_ssl, connection());
+        }
+
         stage(Stage::Connected);
         return Result::Again;
       }
@@ -365,6 +418,17 @@ ConnectProcessor::from_pool() {
       // socket is dead. try the next one.
       return Result::Again;
     }
+
+    if (auto *ev = trace_event_socket_from_pool_) {
+      ev->attrs.emplace_back("mysql.error_message", "no match");
+      trace_span_end(ev, TraceEvent::StatusCode::kError);
+    }
+
+  } else {
+    if (auto *ev = trace_event_socket_from_pool_) {
+      ev->attrs.emplace_back("mysql.error_message", "no pool");
+      trace_span_end(ev, TraceEvent::StatusCode::kError);
+    }
   }
 
   stage(Stage::Connect);
@@ -376,6 +440,15 @@ stdx::expected<Processor::Result, std::error_code> ConnectProcessor::connect() {
     tr.trace(Tracer::Event().stage("connect::connect: " +
                                    mysqlrouter::to_string(server_endpoint_)));
   }
+
+  trace_event_socket_connect_ =
+      trace_span(trace_event_connect_, "mysql/connect");
+
+  if (auto *ev = trace_event_socket_connect_) {
+    ev->attrs.emplace_back("net.peer.name", endpoints_it_->host_name());
+    ev->attrs.emplace_back("net.peer.port", endpoints_it_->service_name());
+  }
+
 #if 0
   if (log_level_is_handled(mysql_harness::logging::LogLevel::kDebug)) {
     log_debug("trying %s", mysqlrouter::to_string(server_endpoint_).c_str());
@@ -442,8 +515,6 @@ stdx::expected<Processor::Result, std::error_code> ConnectProcessor::connect() {
   }
 #endif
 
-  connect_started_ = std::chrono::steady_clock::now();
-
   const auto connect_res = server_sock.connect(server_endpoint_);
 
   // don't assign the connection if disconnect is requested.
@@ -474,16 +545,15 @@ stdx::expected<Processor::Result, std::error_code> ConnectProcessor::connect() {
       // connect in progress, wait for completion.
       stage(Stage::ConnectFinish);
 
+      auto &t = connection()->connect_timer();
+
+      t.expires_after(
+          connection()->context().get_destination_connect_timeout());
+
       if (auto &tr = tracer()) {
         tr.trace(Tracer::Event().stage("connect::wait"));
       }
-
-      auto &timer = connection()->connect_timer();
-
-      timer.expires_after(
-          connection()->context().get_destination_connect_timeout());
-
-      timer.async_wait([this](std::error_code ec) {
+      t.async_wait([this](std::error_code ec) {
         if (ec) return;
 
         if (auto &tr = tracer()) {
@@ -533,20 +603,8 @@ stdx::expected<Processor::Result, std::error_code> ConnectProcessor::connect() {
   return Result::Again;
 }
 
-namespace {
-std::string pretty_endpoint(const net::ip::tcp::endpoint &ep,
-                            const std::string &hostname) {
-  if (ep.address().to_string() == hostname) return mysqlrouter::to_string(ep);
-
-  return mysqlrouter::to_string(ep) + " /* " + hostname + " */";
-}
-}  // namespace
-
 stdx::expected<Processor::Result, std::error_code>
 ConnectProcessor::connect_finish() {
-  auto connect_duration = std::chrono::duration_cast<std::chrono::milliseconds>(
-      std::chrono::steady_clock::now() - connect_started_);
-
   connection()->connect_timer().cancel();
 
   auto &server_conn = connection()->socket_splicer()->server_conn();
@@ -554,51 +612,29 @@ ConnectProcessor::connect_finish() {
   // cancel all handlers.
   (void)server_conn.cancel();
 
-  if (auto ec = connection()->connect_error_code()) {
-    log_debug("connect(%s, %d) failed: %s:%s",
-              server_endpoint_.address().to_string().c_str(),
-              server_endpoint_.port(), ec.category().name(),
-              ec.message().c_str());
+  if (connection()->connect_error_code() != std::error_code{}) {
+    last_ec_ = connection()->connect_error_code();
+
+    (void)server_conn.close();
 
     if (auto &tr = tracer()) {
-      tr.trace(
-          Tracer::Event().stage("connect::connect_finish: " + ec.message()));
+      tr.trace(Tracer::Event().stage("connect::connect_finish: " +
+                                     last_ec_.message()));
     }
 
-    connect_errors_.emplace_back(
-        "connect(" +
-            pretty_endpoint(server_endpoint_, (*destinations_it_)->hostname()) +
-            ") failed after " + std::to_string(connect_duration.count()) + "ms",
-        ec);
-
-    destination_ec_ = ec;
-
     stage(Stage::NextEndpoint);
     return Result::Again;
   }
 
   auto sock_ec_res = sock_error_code(server_conn);
   if (!sock_ec_res) {
-    auto ec = sock_ec_res.error();
-
-    log_debug("connect(%s, %d) failed: %s:%s",
-              server_endpoint_.address().to_string().c_str(),
-              server_endpoint_.port(), ec.category().name(),
-              ec.message().c_str());
+    last_ec_ = sock_ec_res.error();
 
     if (auto &tr = tracer()) {
-      tr.trace(
-          Tracer::Event().stage("connect::connect_finish: " + ec.message()));
+      tr.trace(Tracer::Event().stage("connect::connect_finish: " +
+                                     last_ec_.message()));
     }
 
-    connect_errors_.emplace_back(
-        "connect(" +
-            pretty_endpoint(server_endpoint_, (*destinations_it_)->hostname()) +
-            ")::getsockopt()",
-        ec);
-
-    destination_ec_ = ec;
-
     stage(Stage::NextEndpoint);
     return Result::Again;
   }
@@ -606,38 +642,36 @@ ConnectProcessor::connect_finish() {
   auto sock_ec = *sock_ec_res;
 
   if (sock_ec != std::error_code{}) {
-    log_debug("connect(%s, %d) failed: %s:%s",
-              server_endpoint_.address().to_string().c_str(),
-              server_endpoint_.port(), sock_ec.category().name(),
-              sock_ec.message().c_str());
+    last_ec_ = sock_ec;
 
     if (auto &tr = tracer()) {
       tr.trace(Tracer::Event().stage("connect::connect_finish: " +
-                                     sock_ec.message()));
+                                     last_ec_.message()));
     }
 
-    connect_errors_.emplace_back(
-        "connect(" +
-            pretty_endpoint(server_endpoint_, (*destinations_it_)->hostname()) +
-            ") failed after " + std::to_string(connect_duration.count()) + "ms",
-        sock_ec);
-
-    destination_ec_ = sock_ec;
-
     stage(Stage::NextEndpoint);
     return Result::Again;
   }
 
+  if (auto *ev = trace_event_socket_connect_) {
+    trace_span_end(ev);
+  }
+
   stage(Stage::Connected);
   return Result::Again;
 }
 
 stdx::expected<Processor::Result, std::error_code>
 ConnectProcessor::next_endpoint() {
-  (void)connection()->socket_splicer()->server_conn().close();
-
   if (auto &tr = tracer()) {
-    tr.trace(Tracer::Event().stage("connect::next_endpoint"));
+    tr.trace(
+        Tracer::Event().stage("connect::next_endpoint: " + last_ec_.message()));
+  }
+
+  if (auto *ev = trace_event_socket_connect_) {
+    ev->attrs.emplace_back("mysql.error_message", last_ec_.message());
+
+    trace_span_end(ev);
   }
 
   std::advance(endpoints_it_, 1);
@@ -645,32 +679,27 @@ ConnectProcessor::next_endpoint() {
   if (endpoints_it_ != endpoints_.end()) {
     stage(Stage::InitConnect);
     return Result::Again;
-  }
+  } else {
+    auto &destination = *destinations_it_;
 
-  // no more endpoints for this destination.
+    // report back the connect status to the destination
+    destination->connect_status(last_ec_);
 
-  auto &destination = *destinations_it_;
+    if (last_ec_) {
+      auto hostname = destination->hostname();
+      auto port = destination->port();
 
-  // report back the connect status to the destination
-  destination->connect_status(destination_ec_);
+      auto &ctx = connection()->context();
 
-  if (destination_ec_) {
-    auto hostname = destination->hostname();
-    auto port = destination->port();
-
-    auto &ctx = connection()->context();
-
-    if (ctx.shared_quarantine().update({hostname, port}, false)) {
-      log_debug("[%s] add destination '%s:%d' to quarantine",
-                ctx.get_name().c_str(), hostname.c_str(), port);
-    } else {
-      // failed to connect, but not quarantined. Don't close the ports, yet.
-      all_quarantined_ = false;
+      if (ctx.shared_quarantine().update({hostname, port}, false)) {
+        log_debug("[%s] add destination '%s:%d' to quarantine",
+                  ctx.get_name().c_str(), hostname.c_str(), port);
+      }
     }
-  }
 
-  stage(Stage::NextDestination);
-  return Result::Again;
+    stage(Stage::NextDestination);
+    return Result::Again;
+  }
 }
 
 bool ConnectProcessor::is_destination_good(const std::string &hostname,
@@ -701,39 +730,42 @@ ConnectProcessor::next_destination() {
 
     const auto &destination = *destinations_it_;
 
+    // for read-only connections, skip the writable destinations,
+    // for read-write connections, skip the read-only destinations.
+    if (skip_destination(connection(), destination.get())) continue;
+
     if (is_destination_good(destination->hostname(), destination->port())) {
       break;
     }
-
-    connect_errors_.emplace_back(
-        "connect(/* " + destination->hostname() + ":" +
-            std::to_string(destination->port()) + " */)",
-        make_error_code(DestinationsErrc::kQuarantined));
   } while (true);
 
   if (destinations_it_ != destinations_.end()) {
     // next destination
     stage(Stage::Resolve);
     return Result::Again;
+  } else {
+    auto refresh_res =
+        connection()->destinations()->refresh_destinations(destinations_);
+    if (refresh_res) {
+      destinations_ = std::move(refresh_res.value());
+
+      stage(Stage::InitDestination);
+      return Result::Again;
+    } else if (connection()->context().access_mode() ==
+                   routing::AccessMode::kAuto &&
+               connection()->expected_server_mode() ==
+                   mysqlrouter::ServerMode::ReadOnly) {
+      // if we want a RO connections but there are only primaries, take a
+      // primary.
+      connection()->expected_server_mode(mysqlrouter::ServerMode::ReadWrite);
+      stage(Stage::InitDestination);
+      return Result::Again;
+    } else {
+      // we couldn't connect to any of the destinations. Give up.
+      stage(Stage::Error);
+      return Result::Again;
+    }
   }
-
-  // no more destinations.
-
-  if (auto refresh_res =
-          connection()->destinations()->refresh_destinations(destinations_)) {
-    destinations_ = std::move(refresh_res.value());
-
-    stage(Stage::InitDestination);
-    return Result::Again;
-  }
-
-  connect_errors_.emplace_back(
-      "end of destinations",
-      make_error_code(DestinationsErrc::kNoDestinations));
-
-  // we couldn't connect to any of the destinations. Give up.
-  stage(Stage::Error);
-  return Result::Again;
 }
 
 stdx::expected<Processor::Result, std::error_code>
@@ -742,15 +774,24 @@ ConnectProcessor::connected() {
     tr.trace(Tracer::Event().stage("connect::connected"));
   }
 
-  // remember the destination we connected too for connection-sharing.
+  if (auto *ev = trace_event_connect_) {
+    trace_span_end(ev);
+  }
+
+  const auto *dest = destinations_it_->get();
+
+  // remember the destination and its server-mode for connection-sharing.
+  if (connection()->expected_server_mode() ==
+      mysqlrouter::ServerMode::Unavailable) {
+    // before the first query, the server-mode is not set, remember it now.
+    connection()->expected_server_mode(dest->server_mode());
+  }
+
   connection()->destination_id(destination_id_from_endpoint(*endpoints_it_));
 
-  {
-    const auto &dest = (*destinations_it_);
-
-    connection()->context().shared_quarantine().update(
-        {dest->hostname(), dest->port()}, true);
-  }
+  // mark destination as reachable.
+  connection()->context().shared_quarantine().update(
+      {dest->hostname(), dest->port()}, true);
 
   // back to the caller.
   stage(Stage::Done);
@@ -758,45 +799,49 @@ ConnectProcessor::connected() {
 }
 
 stdx::expected<Processor::Result, std::error_code> ConnectProcessor::error() {
-  // close the socket if it is still open.
-  (void)connection()->socket_splicer()->server_conn().close();
+  auto *tcp_conn = dynamic_cast<TcpConnection *>(
+      connection()->socket_splicer()->server_conn().connection().get());
+
+  // close socket if it is already open
+  if (tcp_conn != nullptr) (void)tcp_conn->close();
 
   if (auto &tr = tracer()) {
     tr.trace(Tracer::Event().stage("connect::error"));
   }
 
-  const auto last_ec = connect_errors_.back().second;
+  const auto ec = last_ec_;
 
-  connection()->connect_error_code(last_ec);
+  connection()->connect_error_code(ec);
 
-  {
-    std::string msg;
-    for (auto [err, ec] : connect_errors_) {
-      if (!msg.empty()) {
-        msg += ", ";
-      }
-      msg += err;
-      msg += ": ";
-      msg += ec.message();
-    }
-    log_error("[%s] connecting to backend(s) for client from %s failed: %s",
-              connection()->context().get_name().c_str(),
-              connection()->get_client_address().c_str(), msg.c_str());
+  log_debug("ConnectProcessor::error(): %s:%s", ec.category().name(),
+            ec.message().c_str());
+
+  if (ec == DestinationsErrc::kNoDestinations) {
+    log_error("no backend available to connect to");
+  } else {
+    log_fatal_error_code("connecting to backend failed", ec);
   }
 
-  if (last_ec == make_error_condition(std::errc::too_many_files_open) ||
-      last_ec ==
-          make_error_condition(std::errc::too_many_files_open_in_system)) {
+  if (auto *ev = trace_event_connect_) {
+    ev->attrs.emplace_back(
+        "mysql.error_message",
+        ec == DestinationsErrc::kNoDestinations ? "no backend" : ec.message());
+    trace_span_end(ev);
+  }
+
+  if (ec == make_error_condition(std::errc::too_many_files_open) ||
+      ec == make_error_condition(std::errc::too_many_files_open_in_system)) {
     // release file-descriptors on the connection pool when out-of-fds is
     // noticed.
     //
     // don't retry as router may run into an infinite loop.
     ConnectionPoolComponent::get_instance().clear();
-  } else if (connection()->get_destination_id().empty() && all_quarantined_) {
-    // fresh-connect == "destiantion-id is empty"
-
+  } else if (ec == DestinationsErrc::kNoDestinations &&
+             connection()->get_destination_id().empty()) {
     // if there are no destinations for a fresh connect, close the
     // acceptor-ports
+    //
+    // fresh-connect == "destiantion-id is empty"
     if (auto &tr = tracer()) {
       tr.trace(Tracer::Event().stage("connect::error::all_down"));
     }
@@ -806,8 +851,7 @@ stdx::expected<Processor::Result, std::error_code> ConnectProcessor::error() {
         .stop_socket_acceptors();
   }
 
-  connection()->server_protocol()->handshake_state(
-      ClassicProtocolState::HandshakeState::kConnected);
+  connection()->client_greeting_sent(true);
   connection()->authenticated(false);
 
   stage(Stage::Done);
