@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2019, 2021, Oracle and/or its affiliates.
+Copyright (c) 2019, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -28,8 +28,6 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <thread>
 
 #ifdef RAPIDJSON_NO_SIZETYPEDEFINE
-// if we build within the server, it will set RAPIDJSON_NO_SIZETYPEDEFINE
-// globally and require to include my_rapidjson_size_t.h
 #include "my_rapidjson_size_t.h"
 #endif
 
@@ -44,8 +42,9 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include "keyring/keyring_manager.h"
 #include "mock_server_rest_client.h"
 #include "mock_server_testutils.h"
-#include "mysql_session.h"
+#include "mysql/harness/stdx/ranges.h"
 #include "mysqlrouter/cluster_metadata.h"
+#include "mysqlrouter/mysql_session.h"
 #include "mysqlrouter/rest_client.h"
 #include "router_component_test.h"
 #include "router_component_testutils.h"
@@ -55,6 +54,8 @@ using mysqlrouter::ClusterType;
 using mysqlrouter::MySQLSession;
 using namespace std::chrono_literals;
 
+using testing::ElementsAre;
+
 Path g_origin_path;
 
 namespace {
@@ -129,7 +130,7 @@ class AsyncReplicasetTest : public RouterComponentTest {
                       const std::string &routing_section,
                       const std::string &state_file_path,
                       const int expected_errorcode = EXIT_SUCCESS,
-                      std::chrono::milliseconds wait_for_notify_ready = 5s) {
+                      std::chrono::milliseconds wait_for_notify_ready = 30s) {
     const std::string masterkey_file =
         Path(temp_test_dir).join("master.key").str();
     const std::string keyring_file = Path(temp_test_dir).join("keyring").str();
@@ -153,13 +154,19 @@ class AsyncReplicasetTest : public RouterComponentTest {
     return router;
   }
 
-  void set_mock_metadata(uint16_t http_port, const std::string &gr_id,
-                         const std::vector<uint16_t> &gr_node_ports,
-                         unsigned primary_id = 0, unsigned view_id = 0,
+  void set_mock_metadata(uint16_t http_port, const std::string &cluster_id,
+                         const std::vector<uint16_t> &cluster_node_ports,
+                         unsigned primary_id = 0, uint64_t view_id = 0,
                          bool error_on_md_query = false,
-                         bool empty_result_from_cluster_type_query = false) {
-    auto json_doc = mock_GR_metadata_as_json(gr_id, gr_node_ports, primary_id,
-                                             view_id, error_on_md_query);
+                         bool empty_result_from_cluster_type_query = false,
+                         bool is_gr_cluster = false, unsigned gr_pos = 0) {
+    const auto gr_nodes = is_gr_cluster
+                              ? classic_ports_to_gr_nodes(cluster_nodes_ports)
+                              : std::vector<GRNode>{};
+    auto json_doc = mock_GR_metadata_as_json(
+        cluster_id, gr_nodes, gr_pos,
+        classic_ports_to_cluster_nodes(cluster_node_ports), primary_id, view_id,
+        error_on_md_query);
 
     // we can't allow this counter become undefined as that breaks the
     // wait_for_transaction_count_increase logic
@@ -183,8 +190,6 @@ class AsyncReplicasetTest : public RouterComponentTest {
   std::vector<ProcessWrapper *> cluster_nodes;
   std::vector<uint16_t> cluster_nodes_ports;
   std::vector<uint16_t> cluster_http_ports;
-
-  TcpPortPool port_pool_;
 };
 
 const std::chrono::milliseconds AsyncReplicasetTest::kTTL = 50ms;
@@ -219,7 +224,7 @@ TEST_F(AsyncReplicasetTest, NoChange) {
         "// Make our metadata server to return all 3 nodes as a cluster "
         "members");
 
-    // each memeber should report the same view_id (=1)
+    // each member should report the same view_id (=1)
     set_mock_metadata(cluster_http_ports[i], cluster_id, cluster_nodes_ports, 0,
                       view_id);
   }
@@ -227,7 +232,7 @@ TEST_F(AsyncReplicasetTest, NoChange) {
   SCOPED_TRACE("// Create a router state file with all of the members");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE(
       "// Create a configuration file sections with low ttl so that any "
@@ -250,7 +255,8 @@ TEST_F(AsyncReplicasetTest, NoChange) {
       "// Check our state file content, it should not change, there is "
       "single metadata server reported as initially");
 
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 }
 
 /**
@@ -288,7 +294,7 @@ TEST_F(AsyncReplicasetTest, SecondaryAdded) {
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
       create_state_file_content(
-          cluster_id, {cluster_nodes_ports[0], cluster_nodes_ports[1]},
+          cluster_id, "", {cluster_nodes_ports[0], cluster_nodes_ports[1]},
           view_id));
 
   SCOPED_TRACE(
@@ -317,7 +323,7 @@ TEST_F(AsyncReplicasetTest, SecondaryAdded) {
   SCOPED_TRACE(
       "// Check our state file content, it should first contain only 2 "
       "members");
-  check_state_file(state_file, cluster_id,
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
                    {cluster_nodes_ports[0], cluster_nodes_ports[1]}, view_id);
 
   SCOPED_TRACE("// Make a connection to the secondary");
@@ -335,7 +341,8 @@ TEST_F(AsyncReplicasetTest, SecondaryAdded) {
   SCOPED_TRACE(
       "// Check our state file content, it should now contain all 3 members "
       "and increased view_id");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id + 1);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id + 1);
 
   SCOPED_TRACE("// Check that the existing connection is still alive");
   verify_existing_connection_ok(client1.get());
@@ -377,7 +384,7 @@ TEST_F(AsyncReplicasetTest, SecondaryRemovedStillReachable) {
   SCOPED_TRACE("// Create a router state file the 3 members");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE(
       "// Create a configuration file sections with low ttl so that any "
@@ -404,7 +411,8 @@ TEST_F(AsyncReplicasetTest, SecondaryRemovedStillReachable) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should first contain all 3 members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE(
       "// Let's make a connection to the both secondaries, both should be "
@@ -425,7 +433,7 @@ TEST_F(AsyncReplicasetTest, SecondaryRemovedStillReachable) {
   SCOPED_TRACE(
       "// Check our state file content, it should now contain only 2 members "
       "and increased view_id");
-  check_state_file(state_file, cluster_id,
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
                    {cluster_nodes_ports[0], cluster_nodes_ports[1]},
                    view_id + 1);
 
@@ -467,7 +475,7 @@ TEST_F(AsyncReplicasetTest, ClusterIdChanged) {
   SCOPED_TRACE("// Create a router state file with 3 members");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE(
       "// Create a configuration file sections with low ttl so that any "
@@ -494,7 +502,8 @@ TEST_F(AsyncReplicasetTest, ClusterIdChanged) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should first contain all 3 members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Now let's change the md on the PRIMARY: " + cluster_id +
                ", " + std::to_string(view_id) + " (cluster_id, view_id) to " +
@@ -509,7 +518,8 @@ TEST_F(AsyncReplicasetTest, ClusterIdChanged) {
   SCOPED_TRACE(
       "// Check our state file content, not change, the PRIMARYs view of the "
       "world should not get into account as it contains different cluster_id");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 }
 
 /**
@@ -544,7 +554,7 @@ TEST_F(AsyncReplicasetTest, ClusterSecondaryQueryErrors) {
   SCOPED_TRACE("// Create a router state file the 3 members");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE(
       "// Create a configuration file sections with low ttl so that any "
@@ -571,19 +581,22 @@ TEST_F(AsyncReplicasetTest, ClusterSecondaryQueryErrors) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain all 3 members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE(
       "// Check that there are warnings reported for not being able to fetch "
       "the metadata from both secondaries");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
-  const std::string log_content = router.get_full_logfile();
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
+  const std::string log_content = router.get_logfile_content();
 
   for (size_t i = 1; i <= 2; i++) {
     const std::string pattern =
-        "metadata_cache WARNING .* Failed fetching metadata from instance: " +
+        "metadata_cache WARNING .* Failed fetching metadata from metadata "
+        "server on 127.0.0.1:" +
         std::to_string(cluster_nodes_ports[i]);
-    ASSERT_TRUE(pattern_found(log_content, pattern)) << log_content;
+    ASSERT_TRUE(pattern_found(log_content, pattern));
   }
 }
 
@@ -615,7 +628,7 @@ TEST_F(AsyncReplicasetTest, MetadataUnavailableDisconnectFromSecondary) {
   SCOPED_TRACE("// Create a router state file the 3 members");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE(
       "// Create a configuration file. disconnect_on_metadata_unavailable for "
@@ -643,7 +656,8 @@ TEST_F(AsyncReplicasetTest, MetadataUnavailableDisconnectFromSecondary) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain both members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Let's make a connection to the both servers RW and RO");
   auto client1 = make_new_connection_ok(router_port_rw, cluster_nodes_ports[0]);
@@ -668,7 +682,8 @@ TEST_F(AsyncReplicasetTest, MetadataUnavailableDisconnectFromSecondary) {
   SCOPED_TRACE(
       "// Make sure the state file did not change, it should still contain "
       "the 2 members.");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 }
 
 /**
@@ -699,7 +714,7 @@ TEST_F(AsyncReplicasetTest, MetadataUnavailableDisconnectFromPrimary) {
   SCOPED_TRACE("// Create a router state file the 3 members");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE(
       "// Create a configuration file. disconnect_on_metadata_unavailable for "
@@ -727,7 +742,8 @@ TEST_F(AsyncReplicasetTest, MetadataUnavailableDisconnectFromPrimary) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain both members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Let's make a connection to the both servers RW and RO");
   auto client1 = make_new_connection_ok(router_port_rw, cluster_nodes_ports[0]);
@@ -799,9 +815,10 @@ TEST_F(AsyncReplicasetTest, MultipleChangesInTheCluster) {
 
   SCOPED_TRACE(
       "// Let us start with 3 members (one PRIMARY and 2 SECONDARIES)");
-  const std::string state_file = create_state_file(
-      temp_test_dir.name(),
-      create_state_file_content(cluster_id, initial_cluster_members, view_id));
+  const std::string state_file =
+      create_state_file(temp_test_dir.name(),
+                        create_state_file_content(
+                            cluster_id, "", initial_cluster_members, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -825,7 +842,8 @@ TEST_F(AsyncReplicasetTest, MultipleChangesInTheCluster) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, initial_cluster_members, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   initial_cluster_members, view_id);
 
   SCOPED_TRACE("// Now let's mess a little bit with the metadata");
   // let's remove one of the nodes and add another one
@@ -841,7 +859,12 @@ TEST_F(AsyncReplicasetTest, MultipleChangesInTheCluster) {
   ASSERT_TRUE(wait_for_transaction_count_increase(cluster_http_ports[2], 2));
 
   SCOPED_TRACE("// Check that the state file caught up with all those changes");
-  check_state_file(state_file, cluster_id, new_cluster_members, view_id + 1);
+  // since the node 2 became a new PRIMARY it should be first metadata server on
+  // the list now
+  std::vector<uint16_t> md_servers_expected{
+      cluster_nodes_ports[2], cluster_nodes_ports[0], cluster_nodes_ports[3]};
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   md_servers_expected, view_id + 1);
 }
 
 /**
@@ -873,7 +896,7 @@ TEST_F(AsyncReplicasetTest, SecondaryRemoved) {
       "// Let us start with 3 members (one PRIMARY and 2 SECONDARIES)");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -897,7 +920,8 @@ TEST_F(AsyncReplicasetTest, SecondaryRemoved) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Make 2 RO connections, one for each SECONDARY");
   auto client1 = make_new_connection_ok(router_port_ro, cluster_nodes_ports[1]);
@@ -915,7 +939,8 @@ TEST_F(AsyncReplicasetTest, SecondaryRemoved) {
   SCOPED_TRACE(
       "// Check that the state file does not contain the second SECONDARY "
       "anymore");
-  check_state_file(state_file, cluster_id, new_cluster_members, view_id + 1);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   new_cluster_members, view_id + 1);
 
   SCOPED_TRACE(
       "// Check that the existing connection to the second SECONDARY got "
@@ -957,9 +982,10 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldGone) {
                       initial_cluster_members, 0, view_id);
   }
 
-  const std::string state_file = create_state_file(
-      temp_test_dir.name(),
-      create_state_file_content(cluster_id, initial_cluster_members, view_id));
+  const std::string state_file =
+      create_state_file(temp_test_dir.name(),
+                        create_state_file_content(
+                            cluster_id, "", initial_cluster_members, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -983,7 +1009,8 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldGone) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, initial_cluster_members, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   initial_cluster_members, view_id);
 
   SCOPED_TRACE("// Make one RW and one RO connection");
   auto client_rw =
@@ -991,19 +1018,22 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldGone) {
   auto client_ro =
       make_new_connection_ok(router_port_ro, cluster_nodes_ports[1]);
 
-  SCOPED_TRACE("// Now let's remove old primary and add a new one");
+  SCOPED_TRACE(
+      "// Now let's remove old primary and promote a first secondary to become "
+      "one");
   std::vector<uint16_t> new_cluster_members{cluster_nodes_ports[1],
                                             cluster_nodes_ports[2]};
   for (size_t i = 1; i <= 2; i++) {
     set_mock_metadata(cluster_http_ports[i], cluster_id, new_cluster_members,
-                      /*primary_id=*/1, view_id + 1);
+                      /*primary_id=*/0, view_id + 1);
   }
 
   SCOPED_TRACE("// Wait untill the router sees this change");
   ASSERT_TRUE(wait_for_transaction_count_increase(cluster_http_ports[1], 2));
 
   SCOPED_TRACE("// Check that the state file is as expected");
-  check_state_file(state_file, cluster_id, new_cluster_members, view_id + 1);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   new_cluster_members, view_id + 1);
 
   SCOPED_TRACE(
       "// Check that the existing connection to the old PRIMARY got dropped");
@@ -1012,7 +1042,7 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldGone) {
 
   SCOPED_TRACE("// Check that new RW connections is made to the new PRIMARY");
   /*auto client_rw2 =*/make_new_connection_ok(router_port_rw,
-                                              cluster_nodes_ports[2]);
+                                              cluster_nodes_ports[1]);
 }
 
 /**
@@ -1040,7 +1070,7 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldBecomesSecondary) {
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -1064,7 +1094,8 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldBecomesSecondary) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Make one RW and one RO connection");
   auto client_rw =
@@ -1117,7 +1148,7 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldBecomesSecondaryDisconnectOnPromoted) {
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -1143,7 +1174,8 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldBecomesSecondaryDisconnectOnPromoted) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Make one RW and one RO connection");
   auto client_rw =
@@ -1161,7 +1193,12 @@ TEST_F(AsyncReplicasetTest, NewPrimaryOldBecomesSecondaryDisconnectOnPromoted) {
   ASSERT_TRUE(wait_for_transaction_count_increase(cluster_http_ports[1], 2));
 
   SCOPED_TRACE("// Check that the state file is as expected");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id + 1);
+  // since the primary has changed we expect that change reflected in the
+  // metadata-servers order too
+  check_state_file(
+      state_file, ClusterType::RS_V2, cluster_id,
+      {cluster_nodes_ports[1], cluster_nodes_ports[0], cluster_nodes_ports[2]},
+      view_id + 1);
 
   SCOPED_TRACE("// Check that both RW and RO connections are down");
   EXPECT_TRUE(wait_connection_dropped(*client_rw.get()));
@@ -1197,7 +1234,7 @@ TEST_F(AsyncReplicasetTest, OnlyPrimaryLeftAcceptsRWAndRO) {
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -1221,7 +1258,8 @@ TEST_F(AsyncReplicasetTest, OnlyPrimaryLeftAcceptsRWAndRO) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Make one RW and one RO connection");
   auto client_rw =
@@ -1241,8 +1279,8 @@ TEST_F(AsyncReplicasetTest, OnlyPrimaryLeftAcceptsRWAndRO) {
   ASSERT_TRUE(wait_for_transaction_count_increase(cluster_http_ports[1], 2));
 
   SCOPED_TRACE("// Check that the state file is as expected");
-  check_state_file(state_file, cluster_id, {cluster_nodes_ports[1]},
-                   view_id + 1);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   {cluster_nodes_ports[1]}, view_id + 1);
 
   SCOPED_TRACE("// Check that both RW and RO connections are down");
   EXPECT_TRUE(wait_connection_dropped(*client_rw.get()));
@@ -1279,7 +1317,7 @@ TEST_F(AsyncReplicasetTest, OnlyPrimaryLeftAcceptsRW) {
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -1303,7 +1341,8 @@ TEST_F(AsyncReplicasetTest, OnlyPrimaryLeftAcceptsRW) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE("// Make one RO connection");
   auto client_ro =
@@ -1317,8 +1356,8 @@ TEST_F(AsyncReplicasetTest, OnlyPrimaryLeftAcceptsRW) {
   ASSERT_TRUE(wait_for_transaction_count_increase(cluster_http_ports[0], 2));
 
   SCOPED_TRACE("// Check that the state file is as expected");
-  check_state_file(state_file, cluster_id, {cluster_nodes_ports[0]},
-                   view_id + 1);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   {cluster_nodes_ports[0]}, view_id + 1);
 
   SCOPED_TRACE("// Check that RO connection is down and no new is accepted");
   EXPECT_TRUE(wait_connection_dropped(*client_ro.get()));
@@ -1346,7 +1385,7 @@ TEST_P(NodeUnavailableTest, NodeUnavailable) {
   SCOPED_TRACE("// The cluster has 4 nodes but the first SECONDARY is down");
   const auto trace_file =
       get_data_dir().join("metadata_dynamic_nodes_v2_ar.js").str();
-  for (unsigned i = 0, nodes = 0; i < CLUSTER_NODES; ++i) {
+  for (unsigned i = 0; i < CLUSTER_NODES; ++i) {
     if (i == 1) continue;
     cluster_nodes.push_back(&ProcessManager::launch_mysql_server_mock(
         trace_file, cluster_nodes_ports[i], EXIT_SUCCESS, false,
@@ -1355,12 +1394,11 @@ TEST_P(NodeUnavailableTest, NodeUnavailable) {
     SCOPED_TRACE("// All 4 nodes are in the metadata");
     set_mock_metadata(cluster_http_ports[i], cluster_id, cluster_nodes_ports,
                       /*primary_id=*/0, view_id);
-    nodes++;
   }
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -1384,26 +1422,49 @@ TEST_P(NodeUnavailableTest, NodeUnavailable) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE(
       "// Make 3 RO connections, even though one of the secondaries is down "
-      "each of them should be successfull");
-  for (size_t i = 0; i < 3; ++i) {
-    uint16_t expected_port{0};
-    if (routing_strategy == "round-robin" ||
-        routing_strategy == "round-robin-with-fallback") {
-      // for round-robin we should round-robin on nodes with id 2 and 3
-      // (0-based)
-      expected_port = cluster_nodes_ports[2 + i % 2];
-    } else {
-      ASSERT_STREQ(routing_strategy.c_str(), "first-available");
-      // for fiirst-available we should go with the node id = 2 each time as the
-      // node id = 1 is not available
-      expected_port = cluster_nodes_ports[2];
-    }
+      "each of them should be successful");
 
-    /*auto client_ro =*/make_new_connection_ok(router_port_ro, expected_port);
+  std::vector<std::string> connected_ports;
+
+  // make 3 connections.
+  for (int ndx = 0; ndx < 4; ++ndx) {
+    MySQLSession session;
+
+    ASSERT_NO_THROW(session.connect("127.0.0.1", router_port_ro, "username",
+                                    "password", "", ""));
+
+    auto result = session.query_one("select @@port");
+    ASSERT_TRUE(result);
+    ASSERT_NE((*result)[0], nullptr);
+
+    connected_ports.push_back((*result)[0]);
+  }
+
+  if (routing_strategy == "first-available") {
+    auto node_2 = std::to_string(cluster_nodes_ports[2]);
+
+    EXPECT_THAT(connected_ports,
+                ElementsAre(node_2,  //
+                            node_2,  //
+                            node_2,  //
+                            node_2)  //
+    );
+  } else if (routing_strategy == "round-robin" ||
+             routing_strategy == "round-robin-with-fallback") {
+    auto node_2 = std::to_string(cluster_nodes_ports[2]);
+    auto node_3 = std::to_string(cluster_nodes_ports[3]);
+
+    EXPECT_THAT(connected_ports,
+                ElementsAre(node_2,  // try [1], fallover to [2]
+                            node_2,  // use [2]
+                            node_3,  // use [3]
+                            node_2   // use [2]
+                            ));
   }
 }
 
@@ -1445,7 +1506,7 @@ TEST_P(NodeUnavailableAllNodesDownTest, NodeUnavailableAllNodesDown) {
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -1470,7 +1531,8 @@ TEST_P(NodeUnavailableAllNodesDownTest, NodeUnavailableAllNodesDown) {
 
   SCOPED_TRACE(
       "// Check our state file content, it should contain the initial members");
-  check_state_file(state_file, cluster_id, cluster_nodes_ports, view_id);
+  check_state_file(state_file, ClusterType::RS_V2, cluster_id,
+                   cluster_nodes_ports, view_id);
 
   SCOPED_TRACE(
       "// Attempt 2 RO connections, each should fail unless we fallback to the "
@@ -1505,6 +1567,8 @@ class ClusterTypeMismatchTest
  */
 TEST_P(ClusterTypeMismatchTest, ClusterTypeMismatch) {
   const unsigned CLUSTER_NODES = 2;
+  const bool is_gr_cluster =
+      GetParam().tracefile == "metadata_dynamic_nodes_v2_gr.js";
   for (unsigned i = 0; i < CLUSTER_NODES; ++i) {
     cluster_nodes_ports.push_back(port_pool_.get_next_available());
     cluster_http_ports.push_back(port_pool_.get_next_available());
@@ -1520,12 +1584,12 @@ TEST_P(ClusterTypeMismatchTest, ClusterTypeMismatch) {
 
     SCOPED_TRACE("// Let us start with 2 members (PRIMARY and SECONDARY)");
     set_mock_metadata(cluster_http_ports[i], cluster_id, cluster_nodes_ports,
-                      /*primary_id=*/0, view_id);
+                      /*primary_id=*/0, view_id, false, false, is_gr_cluster);
   }
 
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE("// Create a configuration file.");
   const std::string metadata_cache_section =
@@ -1554,9 +1618,8 @@ TEST_P(ClusterTypeMismatchTest, ClusterTypeMismatch) {
                                      "password", "", ""));
 
   SCOPED_TRACE("// Logfile should contain proper message");
-  const std::string log_content = router.get_full_logfile();
-  ASSERT_TRUE(pattern_found(log_content, GetParam().expected_error))
-      << log_content;
+  const std::string log_content = router.get_logfile_content();
+  ASSERT_TRUE(pattern_found(log_content, GetParam().expected_error));
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -1579,6 +1642,7 @@ class UnexpectedResultFromMDRefreshTest
  */
 TEST_P(UnexpectedResultFromMDRefreshTest, UnexpectedResultFromMDRefreshQuery) {
   const unsigned CLUSTER_NODES = 2;
+  const bool is_gr_cluster = GetParam().cluster_type_str == "gr";
   for (unsigned i = 0; i < CLUSTER_NODES; ++i) {
     cluster_nodes_ports.push_back(port_pool_.get_next_available());
     cluster_http_ports.push_back(port_pool_.get_next_available());
@@ -1595,13 +1659,13 @@ TEST_P(UnexpectedResultFromMDRefreshTest, UnexpectedResultFromMDRefreshQuery) {
         "// Make our metadata server to return both nodes as a cluster "
         "members");
     set_mock_metadata(cluster_http_ports[i], cluster_id, cluster_nodes_ports, 0,
-                      view_id);
+                      view_id, false, false, is_gr_cluster, i);
   }
 
   SCOPED_TRACE("// Create a router state file containing both members");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(cluster_id, cluster_nodes_ports, view_id));
+      create_state_file_content(cluster_id, "", cluster_nodes_ports, view_id));
 
   SCOPED_TRACE(
       "// Create a configuration file. disconnect_on_metadata_unavailable for "
@@ -1622,8 +1686,7 @@ TEST_P(UnexpectedResultFromMDRefreshTest, UnexpectedResultFromMDRefreshQuery) {
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   launch_router(temp_test_dir.name(), metadata_cache_section, routing_section,
-                state_file, EXIT_SUCCESS,
-                /*wait_for_notify_ready=*/-1s);
+                state_file);
 
   SCOPED_TRACE("// Wait until the router at least once queried the metadata");
   ASSERT_TRUE(wait_for_transaction_count_increase(cluster_http_ports[0], 2));
@@ -1636,10 +1699,11 @@ TEST_P(UnexpectedResultFromMDRefreshTest, UnexpectedResultFromMDRefreshQuery) {
       "// Make all members to start returning invalid data when queried for "
       "cluster type (empty resultset)");
 
-  for (unsigned i = 0; i < CLUSTER_NODES; ++i) {
-    set_mock_metadata(cluster_http_ports[i], cluster_id, cluster_nodes_ports, 0,
-                      view_id, /*error_on_md_query=*/false,
-                      /*empty_result_from_cluster_type_query=*/true);
+  for (auto [i, http_port] : stdx::views::enumerate(cluster_http_ports)) {
+    set_mock_metadata(http_port, cluster_id, cluster_nodes_ports, 0, view_id,
+                      /*error_on_md_query=*/false,
+                      /*empty_result_from_cluster_type_query=*/true,
+                      is_gr_cluster, i);
   }
 
   SCOPED_TRACE("// Both connections should get dropped");
