@@ -1,17 +1,16 @@
 /*
- Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+ Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
+ This program is also distributed with certain software (including
  but not limited to OpenSSL) that is licensed under separate terms,
  as designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
+ separately licensed software that they have included with MySQL.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -391,26 +390,9 @@ class TestRestApiEnable : public RouterComponentBootstrapTest {
 
  protected:
   void set_globals(std::string cluster_id = "") {
-    auto json_doc = mock_GR_metadata_as_json(
-        cluster_id, {cluster_node_port}, 0, {cluster_node_port},
-        0 /*primary_id*/, 0 /*view_id*/, false /*error_on_md_query*/);
-    JsonAllocator allocator;
-    JsonValue gr_members_json(rapidjson::kArrayType);
-    JsonValue member(rapidjson::kArrayType);
-    member.PushBack(JsonValue("uuid-1", strlen("uuid-1"), allocator),
-                    allocator);
-    member.PushBack(
-        JsonValue(gr_member_ip.c_str(), gr_member_ip.length(), allocator),
-        allocator);
-    member.PushBack(cluster_node_port, allocator);
-    gr_members_json.PushBack(member, allocator);
-
-    json_doc.AddMember("innodb_cluster_instances", gr_members_json, allocator);
-
-    const auto json_str = json_to_string(json_doc);
-
-    EXPECT_NO_THROW(
-        MockServerRestClient(cluster_http_port).set_globals(json_str));
+    set_mock_metadata(cluster_http_port, cluster_id, {cluster_node_port}, 0,
+                      {cluster_node_port}, 0 /*primary_id*/, 0 /*view_id*/,
+                      false /*error_on_md_query*/);
   }
 
   void setup_paths() {
@@ -534,7 +516,11 @@ VeZwMK4Cb8EO7PzsnX2tD6AA5Ums6GhNgYsbJgdq4MdKb3x6YWZ8DpksSIX2
  * WL13906:TS_FR06_01
  */
 TEST_F(TestRestApiEnable, ensure_rest_is_disabled) {
-  do_bootstrap({"--disable-rest", "--client-ssl-mode", "PASSTHROUGH"});
+  do_bootstrap({
+      "--disable-rest",                    //
+      "--client-ssl-mode", "PASSTHROUGH",  //
+      "--disable-rw-split",                //
+  });
 
   EXPECT_FALSE(certificate_files_exists(
       {cert_file_t::k_ca_key, cert_file_t::k_ca_cert, cert_file_t::k_router_key,
@@ -960,12 +946,14 @@ class TestRestApiEnableBootstrapFailover : public TestRestApiEnable {
 
   void setup_mocks(const bool failover_successful) {
     for (auto i = 0; i < k_node_count; ++i) {
-      gr_members.emplace_back(gr_member_ip, port_pool_.get_next_available());
+      const auto port = port_pool_.get_next_available();
+      gr_nodes.emplace_back(port);
+      cluster_nodes.emplace_back(port);
     }
 
     for (auto i = 0; i < k_node_count; ++i) {
       cluster_http_port = port_pool_.get_next_available();
-      auto port = gr_members[i].second;
+      auto port = gr_nodes[i].classic_port;
 
       std::string trace_file;
       if (i == 0 || !failover_successful) {
@@ -985,11 +973,12 @@ class TestRestApiEnableBootstrapFailover : public TestRestApiEnable {
       ASSERT_TRUE(MockServerRestClient(cluster_http_port)
                       .wait_for_rest_endpoint_ready());
 
-      set_mock_bootstrap_data(cluster_http_port, cluster_name, gr_members,
-                              metadata_version, cluster_id);
+      set_mock_metadata(cluster_http_port, cluster_id, gr_nodes, 0,
+                        cluster_nodes, 0, 0, false, gr_member_ip, "",
+                        metadata_version, cluster_name);
     }
 
-    cluster_node_port = gr_members[0].second;
+    cluster_node_port = gr_nodes[0].classic_port;
     router_port_rw = port_pool_.get_next_available();
     router_port_ro = port_pool_.get_next_available();
     router_port_x_rw = port_pool_.get_next_available();
@@ -997,10 +986,11 @@ class TestRestApiEnableBootstrapFailover : public TestRestApiEnable {
   }
 
  private:
-  const mysqlrouter::MetadataSchemaVersion metadata_version{2, 0, 3};
+  const mysqlrouter::MetadataSchemaVersion metadata_version{2, 2, 0};
   const std::string cluster_name{"mycluster"};
   std::vector<std::pair<uint16_t, ProcessWrapper &>> mock_servers;
-  std::vector<std::pair<std::string, unsigned>> gr_members;
+  std::vector<GRNode> gr_nodes;
+  std::vector<ClusterNode> cluster_nodes;
   static const uint8_t k_node_count{3};
 };
 
