@@ -1,17 +1,16 @@
 #ifndef BINLOG_H_INCLUDED
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,16 +27,15 @@
 #include <sys/types.h>
 #include <time.h>
 #include <atomic>
-#include <string_view>
 #include <utility>
 
-#include "libbinlogevents/include/binlog_event.h"  // enum_binlog_checksum_alg
-#include "m_string.h"                              // llstr
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
 #include "my_sharedlib.h"
 #include "my_sys.h"
+#include "mysql/binlog/event/binlog_event.h"  // enum_binlog_checksum_alg
+#include "mysql/binlog/event/trx_boundary_parser.h"
 #include "mysql/components/services/bits/mysql_cond_bits.h"
 #include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/psi_cond_bits.h"
@@ -58,6 +56,7 @@ class Format_description_log_event;
 class Gtid_monitoring_info;
 class Gtid_set;
 class Ha_trx_info;
+class Incident_log_event;
 class Log_event;
 class Master_info;
 class Relay_log_info;
@@ -82,7 +81,7 @@ typedef int64 query_id_t;
 
 /*
   Maximum allowed unique log filename extension for
-  RESET MASTER TO command - 2 Billion
+  RESET BINARY LOGS AND GTIDS command - 2 Billion
  */
 #define MAX_ALLOWED_FN_EXT_RESET_MASTER 2000000000
 
@@ -356,7 +355,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
     (A)    - checksum algorithm descriptor value
     FD.(A) - the value of (A) in FD
   */
-  binary_log::enum_binlog_checksum_alg relay_log_checksum_alg;
+  mysql::binlog::event::enum_binlog_checksum_alg relay_log_checksum_alg;
 
   MYSQL_BIN_LOG(uint *sync_period, bool relay_log = false);
   ~MYSQL_BIN_LOG() override;
@@ -438,9 +437,9 @@ class MYSQL_BIN_LOG : public TC_LOG {
     of all lost GTIDs in the binary log, and stores each set in
     respective argument.
 
-    @param gtid_set Will be filled with all GTIDs in this binary/relay
+    @param all_gtids Will be filled with all GTIDs in this binary/relay
     log.
-    @param lost_groups Will be filled with all GTIDs in the
+    @param lost_gtids Will be filled with all GTIDs in the
     Previous_gtids_log_event of the first binary log that has a
     Previous_gtids_log_event. This is requested to binary logs but not
     to relay logs.
@@ -458,11 +457,11 @@ class MYSQL_BIN_LOG : public TC_LOG {
     @param is_server_starting True if the server is starting.
     @return false on success, true on error.
   */
-  bool init_gtid_sets(Gtid_set *gtid_set, Gtid_set *lost_groups,
-                      bool verify_checksum, bool need_lock,
-                      Transaction_boundary_parser *trx_parser,
-                      Gtid_monitoring_info *partial_trx,
-                      bool is_server_starting = false);
+  bool init_gtid_sets(
+      Gtid_set *all_gtids, Gtid_set *lost_gtids, bool verify_checksum,
+      bool need_lock,
+      mysql::binlog::event::Transaction_boundary_parser *trx_parser,
+      Gtid_monitoring_info *partial_trx, bool is_server_starting = false);
 
   void set_previous_gtid_set_relaylog(Gtid_set *previous_gtid_set_param) {
     assert(is_relay_log);
@@ -601,11 +600,17 @@ class MYSQL_BIN_LOG : public TC_LOG {
     @param[out] total_bytes_var Pointer to variable that will be set to total
                                 number of bytes flushed, or NULL.
 
+    @param[out] rotate_var Pointer to variable that will be set to true if
+                           binlog rotation should be performed after releasing
+                           locks. If rotate is not necessary, the variable will
+                           not be touched.
+
     @param[out] out_queue_var  Pointer to the sessions queue in flush stage.
 
     @return Error code on error, zero on success
   */
-  int process_flush_stage_queue(my_off_t *total_bytes_var, THD **out_queue_var);
+  int process_flush_stage_queue(my_off_t *total_bytes_var, bool *rotate_var,
+                                THD **out_queue_var);
 
   /**
     Flush and commit the transaction.
@@ -703,13 +708,6 @@ class MYSQL_BIN_LOG : public TC_LOG {
 #endif /* defined(MYSQL_SERVER) */
   void add_bytes_written(ulonglong inc) { bytes_written += inc; }
   void reset_bytes_written() { bytes_written = 0; }
-  /// @brief Adds bytes written in the current relay log into the variable
-  /// handling the total number of bytes acquired by the replica. Resets the
-  /// counter of bytes written. If requested by caller,
-  /// acquires relay log space lock
-  /// @param rli Pointer to the applier metadata object
-  /// @param need_log_space_lock Information on whether to acquire the
-  /// lock protecting data responsible for keeping the relay log space at bay
   void harvest_bytes_written(Relay_log_info *rli, bool need_log_space_lock);
   void set_max_size(ulong max_size_arg);
 
@@ -763,7 +761,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
     thread while creating a new relay log file. This should be NULL for
     binary log files.
     @param new_index_number The binary log file index number to start from
-    after the RESET MASTER TO command is called.
+    after the RESET BINARY LOGS AND GTIDS command is called.
   */
   bool open_binlog(const char *log_name, const char *new_name,
                    ulong max_size_arg, bool null_created_arg,
@@ -787,8 +785,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
   */
   bool assign_automatic_gtids_to_flush_group(THD *first_seen);
   bool write_transaction(THD *thd, binlog_cache_data *cache_data,
-                         Binlog_event_writer *writer,
-                         bool parallelization_barrier);
+                         Binlog_event_writer *writer);
 
   /**
      Write a dml into statement cache and then flush it into binlog. It writes
@@ -810,7 +807,12 @@ class MYSQL_BIN_LOG : public TC_LOG {
 
   void report_cache_write_error(THD *thd, bool is_transactional);
   bool check_write_error(const THD *thd);
+  bool write_incident(THD *thd, bool need_lock_log, const char *err_msg,
+                      bool do_flush_and_sync = true);
+  bool write_incident(Incident_log_event *ev, THD *thd, bool need_lock_log,
+                      const char *err_msg, bool do_flush_and_sync = true);
   bool write_event_to_binlog(Log_event *ev);
+  bool write_event_to_binlog_and_flush(Log_event *ev);
   bool write_event_to_binlog_and_sync(Log_event *ev);
   void start_union_events(THD *thd, query_id_t query_id_param);
   void stop_union_events(THD *thd);
@@ -836,22 +838,6 @@ class MYSQL_BIN_LOG : public TC_LOG {
   */
   int write_xa_to_cache(THD *thd);
 
-  ///
-  /// Write an incident and call commit.
-  ///
-  /// The incident is written by marking the transaction cache as having an
-  /// incident and then commit it to binlog. During commit, the flush stage will
-  /// write and Incident_log_event and preceding Gtid_log_event to give the
-  /// incident a proper GTID. The incident will also cause the binlog to be
-  /// rotated and check if some purge is applicable.
-  ///
-  /// @param[in] thd the THD object of current thread.
-  /// @param[in] incident_message A message describing the incident.
-  ///
-  /// @return Returns false if succeeds, otherwise true is returned.
-  ///
-  bool write_incident_commit(THD *thd, std::string_view incident_message);
-
  private:
   bool after_write_to_relay_log(Master_info *mi);
   /**
@@ -952,6 +938,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
   inline char *get_log_fname() { return log_file_name; }
   const char *get_name() const { return name; }
   inline mysql_mutex_t *get_log_lock() { return &LOCK_log; }
+  inline mysql_mutex_t *get_index_lock() { return &LOCK_index; }
   inline mysql_mutex_t *get_commit_lock() { return &LOCK_commit; }
   inline mysql_mutex_t *get_after_commit_lock() { return &LOCK_after_commit; }
   inline mysql_cond_t *get_log_cond() { return &update_cond; }
@@ -1034,6 +1021,11 @@ class MYSQL_BIN_LOG : public TC_LOG {
   */
   int get_gtid_executed(Sid_map *sid_map, Gtid_set *gtid_set);
 
+  /*
+    True while rotating binlog, which is caused by logging Incident_log_event.
+  */
+  bool is_rotating_caused_by_incident;
+
  public:
   /**
     Register LOG_INFO so that log_in_use and adjust_linfo_offsets can
@@ -1148,6 +1140,7 @@ bool mysql_show_binlog_events(THD *thd);
 void check_binlog_cache_size(THD *thd);
 void check_binlog_stmt_cache_size(THD *thd);
 bool binlog_enabled();
+void register_binlog_handler(THD *thd, bool trx);
 int query_error_code(const THD *thd, bool not_killed);
 
 extern const char *log_bin_index;
