@@ -1,16 +1,15 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,18 +25,17 @@
 #include <assert.h>
 #include <algorithm>
 #include <atomic>
-#include <cmath>
-#include <memory>
-#include <optional>
 #include <utility>
 #include <vector>
 
+#include "extra/robin-hood-hashing/robin_hood.h"
 #include "field_types.h"
 #include "my_alloc.h"
 #include "my_bit.h"
+#include "my_xxhash.h"
+
 #include "my_inttypes.h"
 #include "my_sys.h"
-#include "my_xxhash.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysqld_error.h"
 #include "sql/item.h"
@@ -49,6 +47,9 @@
 #include "sql/sql_list.h"
 #include "sql/system_variables.h"
 #include "sql/table.h"
+#include "template_utils.h"
+
+class JOIN;
 
 using hash_join_buffer::LoadBufferRowIntoTableBuffers;
 using hash_join_buffer::LoadImmutableStringIntoTableBuffers;
@@ -66,8 +67,8 @@ HashJoinIterator::HashJoinIterator(
     table_map tables_to_get_rowid_for, size_t max_memory_available,
     const std::vector<HashJoinCondition> &join_conditions,
     bool allow_spill_to_disk, JoinType join_type,
-    const Mem_root_array<Item *> &extra_conditions, bool probe_input_batch_mode,
-    uint64_t *hash_table_generation)
+    const Mem_root_array<Item *> &extra_conditions, HashJoinInput first_input,
+    bool probe_input_batch_mode, uint64_t *hash_table_generation)
     : RowIterator(thd),
       m_state(State::READING_ROW_FROM_PROBE_ITERATOR),
       m_hash_table_generation(hash_table_generation),
@@ -85,6 +86,14 @@ HashJoinIterator::HashJoinIterator(
                         join_conditions.data() + join_conditions.size()),
       m_chunk_files_on_disk(thd->mem_root, kMaxChunks),
       m_estimated_build_rows(estimated_build_rows),
+      // For (LEFT)OUTER and ANTI-join we may have to return rows even if the
+      // build input is empty. Therefore we check the probe input for emptiness
+      // first. If 'probe' is empty, there is no need to read from 'build',
+      // while the converse is not the case.
+      m_first_input(
+          (join_type == JoinType::OUTER || join_type == JoinType::ANTI)
+              ? HashJoinInput::kProbe
+              : first_input),
       m_probe_input_batch_mode(probe_input_batch_mode),
       m_allow_spill_to_disk(allow_spill_to_disk),
       m_join_type(join_type) {
@@ -152,7 +161,7 @@ bool HashJoinIterator::Init() {
   //
   // Note that this only ever happens in the hypergraph optimizer; see comments
   // in CreateIteratorFromAccessPath().
-  if (m_row_buffer.Initialized() &&
+  if (m_row_buffer.inited() &&
       (m_hash_join_type == HashJoinType::IN_MEMORY ||
        (m_hash_join_type == HashJoinType::SPILL_TO_DISK &&
         m_chunk_files_on_disk.empty())) &&
@@ -175,13 +184,15 @@ bool HashJoinIterator::Init() {
     }
   }
 
-  // Prepare to read the build input into the hash map.
-  PrepareForRequestRowId(m_build_input_tables.tables(),
-                         m_tables_to_get_rowid_for);
-  if (m_build_input->Init()) {
-    assert(thd()->is_error() ||
-           thd()->killed);  // my_error should have been called.
-    return true;
+  if (m_first_input == HashJoinInput::kBuild) {
+    // Prepare to read the build input into the hash map.
+    PrepareForRequestRowId(m_build_input_tables.tables(),
+                           m_tables_to_get_rowid_for);
+    if (m_build_input->Init()) {
+      assert(thd()->is_error() ||
+             thd()->killed);  // my_error should have been called.
+      return true;
+    }
   }
 
   // We always start out by doing everything in memory.
@@ -231,10 +242,43 @@ bool HashJoinIterator::Init() {
   PrepareForRequestRowId(m_probe_input_tables.tables(),
                          m_tables_to_get_rowid_for);
 
+  if (m_first_input == HashJoinInput::kProbe) {
+    m_state = State::READING_ROW_FROM_PROBE_ITERATOR;
+
+    if (InitProbeIterator()) {
+      return true;
+    }
+
+    const int result = m_probe_input->Read();
+    if (result == 1) {
+      assert(thd()->is_error() ||
+             thd()->killed);  // my_error should have been called.
+      return true;
+    } else if (result == -1) {
+      m_probe_input->EndPSIBatchModeIfStarted();
+      m_state = State::END_OF_ROWS;
+      return false;
+    } else {
+      assert(result == 0);
+      m_probe_row_read = true;
+      // Prepare to read the build input into the hash map.
+      PrepareForRequestRowId(m_build_input_tables.tables(),
+                             m_tables_to_get_rowid_for);
+      if (m_build_input->Init()) {
+        assert(thd()->is_error() ||
+               thd()->killed);  // my_error should have been called.
+        return true;
+      }
+    }
+  }
+
   // Build the hash table
   if (BuildHashTable()) {
     assert(thd()->is_error() ||
            thd()->killed);  // my_error should have been called.
+    if (m_first_input == HashJoinInput::kProbe) {
+      m_probe_input->EndPSIBatchModeIfStarted();
+    }
     return true;
   }
   if (m_hash_table_generation != nullptr) {
@@ -245,6 +289,9 @@ bool HashJoinIterator::Init() {
     // BuildHashTable() decided that the join is done (the build input is
     // empty, and we are in an inner-/semijoin. Anti-/outer join must output
     // NULL-complemented rows from the probe input).
+    if (m_first_input == HashJoinInput::kProbe) {
+      m_probe_input->EndPSIBatchModeIfStarted();
+    }
     return false;
   }
 
@@ -255,11 +302,14 @@ bool HashJoinIterator::Init() {
     // (We also don't need to read more than one row, but
     // CreateHashJoinAccessPath() has already added a LIMIT 1 for us
     // in this case.)
+    if (m_first_input == HashJoinInput::kProbe) {
+      m_probe_input->EndPSIBatchModeIfStarted();
+    }
     m_state = State::END_OF_ROWS;
     return false;
   }
 
-  return InitProbeIterator();
+  return m_first_input == HashJoinInput::kProbe ? false : InitProbeIterator();
 }
 
 // Construct a join key from a list of join conditions, where the join key from
@@ -567,19 +617,14 @@ bool HashJoinIterator::ReadNextHashJoinChunk() {
   // it works like this; if we are at the end of the build chunk, move to the
   // next. If not, keep reading from the same chunk pair. We also move to the
   // next pair of chunk files if the probe chunk file is empty.
-  bool move_to_next_chunk = false;
-  if (m_current_chunk == -1) {
-    // We are before the first chunk, so move to the next.
-    move_to_next_chunk = true;
-  } else if (m_build_chunk_current_row >=
-             m_chunk_files_on_disk[m_current_chunk].build_chunk.num_rows()) {
-    // We are done reading all the rows from the build chunk.
-    move_to_next_chunk = true;
-  } else if (m_chunk_files_on_disk[m_current_chunk].probe_chunk.num_rows() ==
-             0) {
-    // The probe chunk file is empty.
-    move_to_next_chunk = true;
-  }
+  const bool move_to_next_chunk =
+      // We are before the first chunk, so move to the next.
+      m_current_chunk == -1 ||
+      // We are done reading all the rows from the build chunk.
+      m_build_chunk_current_row >=
+          m_chunk_files_on_disk[m_current_chunk].build_chunk.NumRows() ||
+      // The probe chunk file is empty.
+      m_chunk_files_on_disk[m_current_chunk].probe_chunk.NumRows() == 0;
 
   if (move_to_next_chunk) {
     m_current_chunk++;
@@ -604,7 +649,7 @@ bool HashJoinIterator::ReadNextHashJoinChunk() {
       m_chunk_files_on_disk[m_current_chunk].build_chunk;
 
   const bool reject_duplicate_keys = RejectDuplicateKeys();
-  for (; m_build_chunk_current_row < build_chunk.num_rows();
+  for (; m_build_chunk_current_row < build_chunk.NumRows();
        ++m_build_chunk_current_row) {
     // Read the next row from the chunk file, and put it in the in-memory row
     // buffer. If the buffer goes full, do the probe phase against the rows we
@@ -651,7 +696,7 @@ bool HashJoinIterator::ReadNextHashJoinChunk() {
   m_probe_chunk_current_row = 0;
   SetReadingProbeRowState();
 
-  if (m_build_chunk_current_row < build_chunk.num_rows() &&
+  if (m_build_chunk_current_row < build_chunk.NumRows() &&
       m_join_type != JoinType::INNER) {
     // The build chunk did not fit into memory, causing us to refill the hash
     // table once the probe input is consumed. If we don't take any special
@@ -672,8 +717,9 @@ bool HashJoinIterator::ReadNextHashJoinChunk() {
 
 bool HashJoinIterator::ReadRowFromProbeIterator() {
   assert(m_current_chunk == -1);
+  const int result = m_probe_row_read ? 0 : m_probe_input->Read();
+  m_probe_row_read = false;
 
-  int result = m_probe_input->Read();
   if (result == 1) {
     assert(thd()->is_error() ||
            thd()->killed);  // my_error should have been called.
@@ -754,7 +800,7 @@ bool HashJoinIterator::ReadRowFromProbeChunkFile() {
   // that row into the record buffer of the probe input table.
   HashJoinChunk &current_probe_chunk =
       m_chunk_files_on_disk[m_current_chunk].probe_chunk;
-  if (m_probe_chunk_current_row >= current_probe_chunk.num_rows()) {
+  if (m_probe_chunk_current_row >= current_probe_chunk.NumRows()) {
     // No more rows in the current probe chunk, so load the next chunk of
     // build rows into the hash table.
     if (m_write_to_probe_row_saving) {
@@ -794,7 +840,7 @@ bool HashJoinIterator::ReadRowFromProbeRowSavingFile() {
   // Read one row from the probe row saving file, and put that row into the
   // record buffer of the probe input table.
   if (m_probe_row_saving_read_file_current_row >=
-      m_probe_row_saving_read_file.num_rows()) {
+      m_probe_row_saving_read_file.NumRows()) {
     // We are done reading all the rows from the probe row saving file. If probe
     // row saving is still enabled, we have a new set of rows in the probe row
     // saving write file.
@@ -849,8 +895,11 @@ void HashJoinIterator::LookupProbeRowInHashTable() {
   if (m_join_conditions.empty()) {
     // Skip the call to find() in case we don't have any join conditions.
     // TODO(sgunders): Is this relevant for performance anymore?
-    m_current_row =
-        m_row_buffer.first_row().value_or(LinkedImmutableString{nullptr});
+    if (m_row_buffer.empty()) {
+      m_current_row = LinkedImmutableString{nullptr};
+    } else {
+      m_current_row = m_row_buffer.begin()->second;
+    }
     m_state = State::READING_FIRST_ROW_FROM_HASH_TABLE;
     return;
   }
@@ -877,8 +926,12 @@ void HashJoinIterator::LookupProbeRowInHashTable() {
   hash_join_buffer::Key key{m_temporary_row_and_join_key_buffer.ptr(),
                             m_temporary_row_and_join_key_buffer.length()};
 
-  m_current_row =
-      m_row_buffer.find(key).value_or(LinkedImmutableString{nullptr});
+  auto it = m_row_buffer.find(key);
+  if (it == m_row_buffer.end()) {
+    m_current_row = LinkedImmutableString{nullptr};
+  } else {
+    m_current_row = it->second;
+  }
 
   m_state = State::READING_FIRST_ROW_FROM_HASH_TABLE;
 }
