@@ -1,16 +1,15 @@
-/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -113,7 +112,21 @@ void EstimateSortCost(AccessPath *path) {
                  num_output_rows * std::max(log2(num_output_rows), 1.0));
   }
 
-  path->set_num_output_rows(num_output_rows);
+  if (path->sort().remove_duplicates) {
+    Prealloced_array<const Item *, 4> sort_items(PSI_NOT_INSTRUMENTED);
+    for (const ORDER *order = path->sort().order; order != nullptr;
+         order = order->next) {
+      sort_items.push_back(*order->item);
+    }
+
+    const double aggregate_rows = EstimateAggregateRows(
+        num_input_rows, {sort_items.cbegin(), sort_items.size()}, nullptr);
+
+    path->set_num_output_rows(std::min(num_output_rows, aggregate_rows));
+  } else {
+    path->set_num_output_rows(num_output_rows);
+  }
+
   path->cost = path->init_cost = child->cost + sort_cost;
   path->init_once_cost = 0.0;
   path->num_output_rows_before_filter = path->num_output_rows();
@@ -182,30 +195,36 @@ void EstimateMaterializeCost(THD *thd, AccessPath *path) {
 
   path->set_num_output_rows(0);
   double cost_for_cacheable = 0.0;
-  bool left_block = true;
+  bool left_operand = true;
   subquery_cost = 0.0;
-  for (const MaterializePathParameters::QueryBlock &block :
-       path->materialize().param->query_blocks) {
-    if (block.subquery_path->num_output_rows() >= 0.0) {
+  for (const MaterializePathParameters::Operand &operand :
+       path->materialize().param->m_operands) {
+    if (operand.subquery_path->num_output_rows() >= 0.0) {
       // For INTERSECT and EXCEPT we can never get more rows than we have in
       // the left block, so do not add unless we are looking at left block or
       // we have a UNION.
-      if (left_block || path->materialize().param->table == nullptr ||
+      if (left_operand || path->materialize().param->table == nullptr ||
           path->materialize().param->table->is_union_or_table()) {
         path->set_num_output_rows(path->num_output_rows() +
-                                  block.subquery_path->num_output_rows());
-      } else if (!left_block &&
+                                  operand.subquery_path->num_output_rows());
+      } else if (!left_operand &&
                  path->materialize().param->table->is_intersect()) {
         // INTERSECT can never give more rows than that of its smallest operand
         path->set_num_output_rows(std::min(
-            path->num_output_rows(), block.subquery_path->num_output_rows()));
+            path->num_output_rows(), operand.subquery_path->num_output_rows()));
       }
-      subquery_cost += block.subquery_path->cost;
-      if (block.join != nullptr && block.join->query_block->is_cacheable()) {
-        cost_for_cacheable += block.subquery_path->cost;
+      // For implicit grouping operand.subquery_path->num_output_rows() may be
+      // set (to 1.0) even if operand.subquery_path->cost is undefined (cf.
+      // Bug#35240913).
+      if (operand.subquery_path->cost > 0.0) {
+        subquery_cost += operand.subquery_path->cost;
+        if (operand.join != nullptr &&
+            operand.join->query_block->is_cacheable()) {
+          cost_for_cacheable += operand.subquery_path->cost;
+        }
       }
     }
-    left_block = false;
+    left_operand = false;
   }
 
   if (table_path->type == AccessPath::TABLE_SCAN) {
@@ -260,7 +279,7 @@ void EstimateMaterializeCost(THD *thd, AccessPath *path) {
 namespace {
 
 /// Array of aggregation terms.
-using TermArray = Mem_root_array<const Item *>;
+using TermArray = Bounds_checked_array<const Item *const>;
 
 /**
    This class finds disjoint sets of aggregation terms that form prefixes of
@@ -271,7 +290,7 @@ class AggregateRowEstimator {
  public:
   /// @param terms The aggregation terms.
   /// @param trace Append optimizer trace text to this if non-null.
-  AggregateRowEstimator(const TermArray &terms, string *trace);
+  AggregateRowEstimator(TermArray terms, string *trace);
 
   // No copying of this type.
   AggregateRowEstimator(const AggregateRowEstimator &) = delete;
@@ -322,7 +341,7 @@ class AggregateRowEstimator {
   };
 
   ///  The aggregation terms.
-  const TermArray *m_terms;
+  TermArray m_terms;
 
   /// The set of terms mapped to an index so far.
   MutableOverflowBitset m_consumed_terms;
@@ -336,10 +355,10 @@ class AggregateRowEstimator {
   /// Find an Item_field pointing to 'field' in 'm_terms', if there is one.
   /// @param field The field we look for.
   /// @returns An iterator to the position of 'field' in m_terms, or
-  /// m_terms->cend().
+  /// m_terms.cend().
   TermArray::const_iterator FindField(const Field *field) const {
     return std::find_if(
-        m_terms->cbegin(), m_terms->cend(), [field](const Item *item) {
+        m_terms.cbegin(), m_terms.cend(), [field](const Item *item) {
           assert(field != nullptr);
           return item->type() == Item::FIELD_ITEM &&
                  down_cast<const Item_field *>(item)->field == field;
@@ -347,9 +366,8 @@ class AggregateRowEstimator {
   }
 };
 
-AggregateRowEstimator::AggregateRowEstimator(const TermArray &terms,
-                                             string *trace)
-    : m_terms{&terms},
+AggregateRowEstimator::AggregateRowEstimator(TermArray terms, string *trace)
+    : m_terms{terms},
       m_consumed_terms{current_thd->mem_root, terms.size()},
       m_trace(trace) {
   /* Find keys (indexes) for which:
@@ -396,7 +414,7 @@ AggregateRowEstimator::AggregateRowEstimator(const TermArray &terms,
 double AggregateRowEstimator::MakeNextEstimate() {
   // Pick the longest prefix until we have used all terms or m_prefixes,
   // or until all prefixes have length==0.
-  while (m_terms->size() >
+  while (m_terms.size() >
              static_cast<size_t>(PopulationCount(m_consumed_terms)) &&
          !m_prefixes.empty()) {
     // Find the longest prefix.
@@ -419,7 +437,7 @@ double AggregateRowEstimator::MakeNextEstimate() {
         For each KEY_PART, check if there is still a corresponding aggregation
         item in m_terms.
       */
-      if (IsBitSet(FindField(field) - m_terms->cbegin(), m_consumed_terms)) {
+      if (IsBitSet(FindField(field) - m_terms.cbegin(), m_consumed_terms)) {
         // We did not find it, so it must have been removed when we examined
         // some earlier key. We can thus only use the prefix 0..key_part_no of
         // this key.
@@ -445,7 +463,7 @@ double AggregateRowEstimator::MakeNextEstimate() {
         // row count from a single term.
         m_consumed_terms.SetBit(
             FindField(prefix->m_key->key_part[key_part_no].field) -
-            m_terms->begin());
+            m_terms.begin());
       }
 
       assert(prefix->m_key->records_per_key(prefix->m_length - 1) !=
@@ -529,7 +547,7 @@ string AggregateRowEstimator::Prefix::Print() const {
 @param trace Append optimizer trace text to this if non-null.
 @returns The row estimate for the aggregate operation.
 */
-double EstimateAggregateNoRollupRows(const TermArray &terms, double child_rows,
+double EstimateAggregateNoRollupRows(TermArray terms, double child_rows,
                                      string *trace) {
   // Estimated number of output rows.
   double output_rows = 1.0;
@@ -563,7 +581,7 @@ double EstimateAggregateNoRollupRows(const TermArray &terms, double child_rows,
         (*term)->type() == Item::FIELD_ITEM) {
       const Field *const field = down_cast<const Item_field *>(*term)->field;
       const histograms::Histogram *const histogram =
-          field->table->s->find_histogram(field->field_index());
+          field->table->find_histogram(field->field_index());
 
       double distinct_values;
       if (histogram == nullptr || empty(*histogram)) {
@@ -718,7 +736,7 @@ double EstimateRollupRowsPrimitively(double aggregate_rows,
   @param trace Optimizer trace.
   @return Estimated number of rollup rows.
 */
-double EstimateRollupRowsAdvanced(double aggregate_rows, TermArray &&terms,
+double EstimateRollupRowsAdvanced(double aggregate_rows, TermArray terms,
                                   string *trace) {
   // Make a more accurate rollup row calculation for larger sets.
   double rollup_rows = 1.0;
@@ -760,7 +778,7 @@ double EstimateAggregateRows(const AccessPath *child,
   }
 
   // The aggregation terms.
-  TermArray terms(current_thd->mem_root);
+  Mem_root_array<const Item *> terms(current_thd->mem_root);
   double output_rows;
 
   if (query_block->is_implicitly_grouped()) {
@@ -782,16 +800,8 @@ double EstimateAggregateRows(const AccessPath *child,
       }
     }
 
-    // Do a simple but fast calculation of the row estimate if child_rows is
-    // less than this.
-    constexpr double simple_limit = 10.0;
-
-    output_rows = SmoothTransition(
-        [&](double input_rows) { return std::sqrt(input_rows); },
-        [&](double input_rows) {
-          return EstimateAggregateNoRollupRows(terms, input_rows, trace);
-        },
-        simple_limit, simple_limit * 1.1, child_rows);
+    output_rows = ::EstimateAggregateRows(
+        child_rows, TermArray(terms.data(), terms.size()), trace);
   }
 
   if (rollup) {
@@ -804,8 +814,8 @@ double EstimateAggregateRows(const AccessPath *child,
         },
         [&](double aggregate_rows) {
           assert(terms.size() == term_count);
-          return EstimateRollupRowsAdvanced(aggregate_rows, std::move(terms),
-                                            trace);
+          return EstimateRollupRowsAdvanced(
+              aggregate_rows, TermArray(terms.data(), terms.size()), trace);
         },
         simple_rollup_limit, simple_rollup_limit * 1.1, output_rows);
   }
@@ -815,6 +825,25 @@ double EstimateAggregateRows(const AccessPath *child,
 
 }  // Anonymous namespace.
 
+double EstimateAggregateRows(double child_rows, TermArray aggregate_terms,
+                             string *trace) {
+  if (child_rows < 1.0) {
+    return child_rows;
+  } else {
+    // Do a simple but fast calculation of the row estimate if child_rows is
+    // less than this.
+    constexpr double simple_limit = 10.0;
+
+    return SmoothTransition(
+        [&](double input_rows) { return std::sqrt(input_rows); },
+        [&](double input_rows) {
+          return EstimateAggregateNoRollupRows(aggregate_terms, input_rows,
+                                               trace);
+        },
+        simple_limit, simple_limit * 1.1, child_rows);
+  }
+}
+
 void EstimateAggregateCost(AccessPath *path, const Query_block *query_block,
                            string *trace) {
   const AccessPath *child = path->aggregate().child;
@@ -915,13 +944,3 @@ void EstimateLimitOffsetCost(AccessPath *path) {
                       fraction_start_read * (child->cost - child->init_cost);
   }
 }
-
-void EstimateWindowCost(AccessPath *path) {
-  auto &win = path->window();
-  AccessPath *child = win.child;
-
-  path->set_num_output_rows(child->num_output_rows());
-  path->init_cost = child->init_cost;
-  path->init_once_cost = child->init_once_cost;
-  path->cost = child->cost + kWindowOneRowCost * child->num_output_rows();
-}
