@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -34,13 +34,14 @@
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_io.h"
-#include "my_loglevel.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
 #include "sql/current_thd.h"
@@ -97,14 +98,12 @@ Delegate::Delegate(
 #else
   if (mysql_rwlock_init(0, &lock)) return;
 #endif
-  init_sql_alloc(key_memory_delegate, &memroot, 1024, 0);
   inited = true;
 }
 
 Delegate::~Delegate() {
   inited = false;
   mysql_rwlock_destroy(&lock);
-  free_root(&memroot, MYF(0));
 }
 
 int Delegate::add_observer(void *observer, st_plugin_int *plugin) {
@@ -205,11 +204,9 @@ void Delegate::update_plugin_ref_count() {
   int intern_value = m_configured_lock_type.load();
 
   if (intern_value == DELEGATE_SPIN_LOCK && opt_value == DELEGATE_OS_LOCK) {
-    for (std::map<plugin_ref, size_t>::iterator ref =
-             m_acquired_references.begin();
-         ref != m_acquired_references.end(); ++ref) {
-      for (size_t count = ref->second; count != 0; --count)
-        plugin_unlock(NULL, ref->first);
+    for (auto ref : m_acquired_references) {
+      for (size_t count = ref.second; count != 0; --count)
+        plugin_unlock(nullptr, ref.first);
     }
     m_acquired_references.clear();
   } else if (intern_value == DELEGATE_OS_LOCK &&
@@ -246,7 +243,7 @@ bool Delegate::use_spin_lock_type() {
 }
 
 void Delegate::acquire_plugin_ref_count(Observer_info *info) {
-  plugin_ref internal_ref = plugin_lock(NULL, &info->plugin);
+  plugin_ref internal_ref = plugin_lock(nullptr, &info->plugin);
   ++(m_acquired_references[internal_ref]);
 }
 
@@ -392,6 +389,15 @@ int delegates_init() {
   return 0;
 }
 
+void delegates_shutdown() {
+  if (opt_replication_optimize_for_static_plugin_config) {
+    opt_replication_optimize_for_static_plugin_config = false;
+    delegates_acquire_locks();
+    delegates_update_lock_type();
+    delegates_release_locks();
+  }
+}
+
 void delegates_destroy() {
   if (transaction_delegate) transaction_delegate->~Trans_delegate();
   if (binlog_storage_delegate)
@@ -408,32 +414,26 @@ static void delegates_update_plugin_ref_count() {
   if (binlog_storage_delegate)
     binlog_storage_delegate->update_plugin_ref_count();
   if (server_state_delegate) server_state_delegate->update_plugin_ref_count();
-#ifdef HAVE_REPLICATION
   if (binlog_transmit_delegate)
     binlog_transmit_delegate->update_plugin_ref_count();
   if (binlog_relay_io_delegate)
     binlog_relay_io_delegate->update_plugin_ref_count();
-#endif /* HAVE_REPLICATION */
 }
 
 void delegates_acquire_locks() {
   if (transaction_delegate) transaction_delegate->write_lock();
   if (binlog_storage_delegate) binlog_storage_delegate->write_lock();
   if (server_state_delegate) server_state_delegate->write_lock();
-#ifdef HAVE_REPLICATION
   if (binlog_transmit_delegate) binlog_transmit_delegate->write_lock();
   if (binlog_relay_io_delegate) binlog_relay_io_delegate->write_lock();
-#endif /* HAVE_REPLICATION */
 }
 
 void delegates_release_locks() {
   if (transaction_delegate) transaction_delegate->unlock();
   if (binlog_storage_delegate) binlog_storage_delegate->unlock();
   if (server_state_delegate) server_state_delegate->unlock();
-#ifdef HAVE_REPLICATION
   if (binlog_transmit_delegate) binlog_transmit_delegate->unlock();
   if (binlog_relay_io_delegate) binlog_relay_io_delegate->unlock();
-#endif /* HAVE_REPLICATION */
 }
 
 void delegates_update_lock_type() {
@@ -442,10 +442,8 @@ void delegates_update_lock_type() {
   if (transaction_delegate) transaction_delegate->update_lock_type();
   if (binlog_storage_delegate) binlog_storage_delegate->update_lock_type();
   if (server_state_delegate) server_state_delegate->update_lock_type();
-#ifdef HAVE_REPLICATION
   if (binlog_transmit_delegate) binlog_transmit_delegate->update_lock_type();
   if (binlog_relay_io_delegate) binlog_relay_io_delegate->update_lock_type();
-#endif /* HAVE_REPLICATION */
 }
 
 /*
@@ -456,93 +454,98 @@ void delegates_update_lock_type() {
   Add observer plugins to the thd->lex list, after each statement, all
   plugins add to thd->lex will be automatically unlocked.
  */
-#define FOREACH_OBSERVER(r, f, args)                                   \
-  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);       \
-  read_lock();                                                         \
-  Observer_info_iterator iter = observer_info_iter();                  \
-  Observer_info *info = iter++;                                        \
-  bool replication_optimize_for_static_plugin_config =                 \
-      this->use_spin_lock_type();                                      \
-  for (; info; info = iter++) {                                        \
-    plugin_ref plugin = (replication_optimize_for_static_plugin_config \
-                             ? info->plugin                            \
-                             : my_plugin_lock(0, &info->plugin));      \
-    if (!plugin) {                                                     \
-      /* plugin is not intialized or deleted, this is not an error */  \
-      r = 0;                                                           \
-      break;                                                           \
-    }                                                                  \
-    if (!replication_optimize_for_static_plugin_config)                \
-      plugins.push_back(plugin);                                       \
-    if (((Observer *)info->observer)->f &&                             \
-        ((Observer *)info->observer)->f args) {                        \
-      r = 1;                                                           \
-      LogEvent()                                                       \
-          .prio(ERROR_LEVEL)                                           \
-          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                      \
-          .subsys(LOG_SUBSYSTEM_TAG)                                   \
-          .function(#f)                                                \
-          .message("Run function '" #f "' in plugin '%s' failed",      \
-                   info->plugin_int->name.str);                        \
-      break;                                                           \
-    }                                                                  \
-  }                                                                    \
-  unlock();                                                            \
-  /*                                                                   \
-     Unlock plugins should be done after we released the Delegate lock \
-     to avoid possible deadlock when this is the last user of the      \
-     plugin, and when we unlock the plugin, it will try to             \
-     deinitialize the plugin, which will try to lock the Delegate in   \
-     order to remove the observers.                                    \
-  */                                                                   \
-  if (!plugins.empty()) plugin_unlock_list(0, &plugins[0], plugins.size());
+#define FOREACH_OBSERVER(r, f, args)                                    \
+  r = 0;                                                                \
+  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);        \
+  read_lock();                                                          \
+  Observer_info_iterator iter = observer_info_iter();                   \
+  Observer_info *info = iter++;                                         \
+  bool replication_optimize_for_static_plugin_config =                  \
+      this->use_spin_lock_type();                                       \
+  for (; info; info = iter++) {                                         \
+    plugin_ref plugin = (replication_optimize_for_static_plugin_config  \
+                             ? info->plugin                             \
+                             : my_plugin_lock(nullptr, &info->plugin)); \
+    if (!plugin) {                                                      \
+      /* plugin is not initialized or deleted, this is not an error */  \
+      continue;                                                         \
+    }                                                                   \
+    if (!replication_optimize_for_static_plugin_config)                 \
+      plugins.push_back(plugin);                                        \
+    if (((Observer *)info->observer)->f &&                              \
+        ((Observer *)info->observer)->f args) {                         \
+      r = 1;                                                            \
+      LogEvent()                                                        \
+          .prio(ERROR_LEVEL)                                            \
+          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                       \
+          .subsys(LOG_SUBSYSTEM_TAG)                                    \
+          .function(#f)                                                 \
+          .message("Run function '" #f "' in plugin '%s' failed",       \
+                   info->plugin_int->name.str);                         \
+      break;                                                            \
+    }                                                                   \
+  }                                                                     \
+  unlock();                                                             \
+  /*                                                                    \
+     Unlock plugins should be done after we released the Delegate lock  \
+     to avoid possible deadlock when this is the last user of the       \
+     plugin, and when we unlock the plugin, it will try to              \
+     deinitialize the plugin, which will try to lock the Delegate in    \
+     order to remove the observers.                                     \
+  */                                                                    \
+  if (!plugins.empty())                                                 \
+    plugin_unlock_list(nullptr, &plugins[0], plugins.size());
 
-#define FOREACH_OBSERVER_ERROR_OUT(r, f, args, out)                    \
-  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);       \
-  read_lock();                                                         \
-  Observer_info_iterator iter = observer_info_iter();                  \
-  Observer_info *info = iter++;                                        \
-                                                                       \
-  bool replication_optimize_for_static_plugin_config =                 \
-      this->use_spin_lock_type();                                      \
-  int error_out = 0;                                                   \
-  for (; info; info = iter++) {                                        \
-    plugin_ref plugin = (replication_optimize_for_static_plugin_config \
-                             ? info->plugin                            \
-                             : my_plugin_lock(0, &info->plugin));      \
-    if (!plugin) {                                                     \
-      /* plugin is not intialized or deleted, this is not an error */  \
-      r = 0;                                                           \
-      break;                                                           \
-    }                                                                  \
-    if (!replication_optimize_for_static_plugin_config)                \
-      plugins.push_back(plugin);                                       \
-                                                                       \
-    bool hook_error = false;                                           \
-    hook_error = ((Observer *)info->observer)->f(args, error_out);     \
-                                                                       \
-    out += error_out;                                                  \
-    if (hook_error) {                                                  \
-      r = 1;                                                           \
-      LogEvent()                                                       \
-          .prio(ERROR_LEVEL)                                           \
-          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                      \
-          .subsys(LOG_SUBSYSTEM_TAG)                                   \
-          .function(#f)                                                \
-          .message("Run function '" #f "' in plugin '%s' failed",      \
-                   info->plugin_int->name.str);                        \
-      break;                                                           \
-    }                                                                  \
-  }                                                                    \
-  unlock();                                                            \
-  /*                                                                   \
-     Unlock plugins should be done after we released the Delegate lock \
-     to avoid possible deadlock when this is the last user of the      \
-     plugin, and when we unlock the plugin, it will try to             \
-     deinitialize the plugin, which will try to lock the Delegate in   \
-     order to remove the observers.                                    \
-  */                                                                   \
-  if (!plugins.empty()) plugin_unlock_list(0, &plugins[0], plugins.size());
+#define FOREACH_OBSERVER_ERROR_OUT(r, f, args, out)                     \
+  r = 0;                                                                \
+  Prealloced_array<plugin_ref, 8> plugins(PSI_NOT_INSTRUMENTED);        \
+  read_lock();                                                          \
+  Observer_info_iterator iter = observer_info_iter();                   \
+  Observer_info *info = iter++;                                         \
+                                                                        \
+  bool replication_optimize_for_static_plugin_config =                  \
+      this->use_spin_lock_type();                                       \
+  int error_out = 0;                                                    \
+  for (; info; info = iter++) {                                         \
+    plugin_ref plugin = (replication_optimize_for_static_plugin_config  \
+                             ? info->plugin                             \
+                             : my_plugin_lock(nullptr, &info->plugin)); \
+    if (!plugin) {                                                      \
+      /* plugin is not initialized or deleted, this is not an error */  \
+      continue;                                                         \
+    }                                                                   \
+    if (!replication_optimize_for_static_plugin_config)                 \
+      plugins.push_back(plugin);                                        \
+                                                                        \
+    if (nullptr == ((Observer *)info->observer)->f) {                   \
+      continue;                                                         \
+    }                                                                   \
+    bool hook_error = false;                                            \
+    hook_error = ((Observer *)info->observer)->f(args, error_out);      \
+                                                                        \
+    out += error_out;                                                   \
+    if (hook_error) {                                                   \
+      r = 1;                                                            \
+      LogEvent()                                                        \
+          .prio(ERROR_LEVEL)                                            \
+          .errcode(ER_RPL_PLUGIN_FUNCTION_FAILED)                       \
+          .subsys(LOG_SUBSYSTEM_TAG)                                    \
+          .function(#f)                                                 \
+          .message("Run function '" #f "' in plugin '%s' failed",       \
+                   info->plugin_int->name.str);                         \
+      break;                                                            \
+    }                                                                   \
+  }                                                                     \
+  unlock();                                                             \
+  /*                                                                    \
+     Unlock plugins should be done after we released the Delegate lock  \
+     to avoid possible deadlock when this is the last user of the       \
+     plugin, and when we unlock the plugin, it will try to              \
+     deinitialize the plugin, which will try to lock the Delegate in    \
+     order to remove the observers.                                     \
+  */                                                                    \
+  if (!plugins.empty())                                                 \
+    plugin_unlock_list(nullptr, &plugins[0], plugins.size());
 
 static bool se_before_commit(THD *, plugin_ref plugin, void *arg) {
   handlerton *hton = plugin_data<handlerton *>(plugin);
@@ -577,12 +580,45 @@ int Trans_delegate::before_commit(THD *thd, bool all,
   param.is_create_table_as_query_block =
       (thd->lex->sql_command == SQLCOM_CREATE_TABLE &&
        !thd->lex->query_block->field_list_is_empty());
+  param.thd = thd;
 
   bool is_real_trans =
       (all || !thd->get_transaction()->is_active(Transaction_ctx::SESSION));
   if (is_real_trans) param.flags |= TRANS_IS_REAL_TRANS;
 
   int ret = 0;
+
+  /* After this debug point we mark the transaction as committing in THD. */
+  DBUG_EXECUTE_IF("trans_delegate_before_commit_before_before_call_observers", {
+    const char act[] =
+        "now signal "
+        "signal.trans_delegate_before_commit_before_before_call_observers_"
+        "reached "
+        "wait_for "
+        "signal.trans_delegate_before_commit_before_before_call_observers_"
+        "waiting";
+    assert(!debug_sync_set_action(thd, STRING_WITH_LEN(act)));
+  });
+
+  thd->rpl_thd_ctx.set_tx_rpl_delegate_stage_status(
+      Rpl_thd_context::TX_RPL_STAGE_BEFORE_COMMIT);
+
+  /**
+    If thread is killed or commits are blocked do not commit the transaction.
+    Post this thread cannot be killed.
+  */
+  if (thd->is_killed() || m_rollback_transaction_not_reached_before_commit) {
+    /**
+      Disconnect the client connection if not already done.
+      Do not KILL connection if the transaction is going to be rolledback.
+    */
+    if (!thd->is_killed()) {
+      mysql_mutex_lock(&thd->LOCK_thd_data);
+      thd->awake(THD::KILL_CONNECTION);
+      mysql_mutex_unlock(&thd->LOCK_thd_data);
+    }
+    return 1;
+  }
   FOREACH_OBSERVER(ret, before_commit, (&param));
   plugin_foreach(thd, se_before_commit, MYSQL_STORAGE_ENGINE_PLUGIN, &param);
   return ret;
@@ -755,6 +791,8 @@ int Trans_delegate::before_rollback(THD *thd, bool all) {
   if (is_real_trans) param.flags |= TRANS_IS_REAL_TRANS;
 
   int ret = 0;
+  thd->rpl_thd_ctx.set_tx_rpl_delegate_stage_status(
+      Rpl_thd_context::TX_RPL_STAGE_BEFORE_ROLLBACK);
   FOREACH_OBSERVER(ret, before_rollback, (&param));
   plugin_foreach(thd, se_before_rollback, MYSQL_STORAGE_ENGINE_PLUGIN, &param);
   return ret;
@@ -778,6 +816,9 @@ int Trans_delegate::after_commit(THD *thd, bool all) {
   param.gtid_info.sidno = gtid.sidno;
   param.gtid_info.gno = gtid.gno;
 
+  thd->rpl_thd_ctx.last_used_gtid_tracker_ctx().get_last_used_sid(
+      param.gtid_info.sid);
+
   bool is_real_trans =
       (all || !thd->get_transaction()->is_active(Transaction_ctx::SESSION));
   if (is_real_trans) param.flags |= TRANS_IS_REAL_TRANS;
@@ -817,6 +858,11 @@ int Trans_delegate::after_rollback(THD *thd, bool all) {
 
 int Trans_delegate::trans_begin(THD *thd, int &out) {
   DBUG_TRACE;
+  if (m_rollback_transaction_on_begin) {
+    out = ER_OPERATION_NOT_ALLOWED_WHILE_PRIMARY_CHANGE_IS_RUNNING;
+    return 0;
+  }
+
   Trans_param param;
   TRANS_PARAM_ZERO(param);
   param.server_uuid = server_uuid;
@@ -826,12 +872,39 @@ int Trans_delegate::trans_begin(THD *thd, int &out) {
   param.hold_timeout = thd->variables.net_wait_timeout;
   param.server_id = thd->server_id;
   param.rpl_channel_type = thd->rpl_thd_ctx.get_rpl_channel_type();
+  param.thd = thd;
 
   int ret = 0;
+  thd->rpl_thd_ctx.set_tx_rpl_delegate_stage_status(
+      Rpl_thd_context::TX_RPL_STAGE_BEGIN);
   FOREACH_OBSERVER_ERROR_OUT(ret, begin, &param, out);
   return ret;
 }
 
+int Trans_delegate::set_transactions_at_begin_must_fail() {
+  DBUG_TRACE;
+  m_rollback_transaction_on_begin = true;
+  return false;
+}
+
+int Trans_delegate::set_no_restrictions_at_transaction_begin() {
+  DBUG_TRACE;
+  m_rollback_transaction_on_begin = false;
+  return false;
+}
+
+int Trans_delegate::set_transactions_not_reached_before_commit_must_fail() {
+  DBUG_TRACE;
+  m_rollback_transaction_not_reached_before_commit = true;
+  return false;
+}
+
+int Trans_delegate::set_no_restrictions_at_transactions_before_commit() {
+  DBUG_TRACE;
+  m_rollback_transaction_not_reached_before_commit = false;
+  return false;
+}
+
 int Binlog_storage_delegate::after_flush(THD *thd, const char *log_file,
                                          my_off_t log_pos) {
   DBUG_TRACE;
@@ -1101,6 +1174,8 @@ void Binlog_relay_IO_delegate::init_param(Binlog_relay_IO_param *param,
   param->port = mi->port;
   param->master_log_name = const_cast<char *>(mi->get_master_log_name());
   param->master_log_pos = mi->get_master_log_pos();
+  param->source_connection_auto_failover =
+      mi->is_source_connection_auto_failover();
 }
 
 int Binlog_relay_IO_delegate::thread_start(THD *thd, Master_info *mi) {
@@ -1280,7 +1355,40 @@ int unregister_binlog_relay_io_observer(Binlog_relay_IO_observer *observer,
   return binlog_relay_io_delegate->remove_observer(observer);
 }
 
-int launch_hook_trans_begin(THD *thd, TABLE_LIST *all_tables) {
+static bool is_show_status(enum_sql_command sql_command) {
+  switch (sql_command) {
+    case SQLCOM_SHOW_VARIABLES:
+    case SQLCOM_SHOW_STATUS:
+    case SQLCOM_SHOW_ENGINE_LOGS:
+    case SQLCOM_SHOW_ENGINE_STATUS:
+    case SQLCOM_SHOW_ENGINE_MUTEX:
+    case SQLCOM_SHOW_PROCESSLIST:
+    case SQLCOM_SHOW_MASTER_STAT:
+    case SQLCOM_SHOW_SLAVE_STAT:
+    case SQLCOM_SHOW_CHARSETS:
+    case SQLCOM_SHOW_COLLATIONS:
+    case SQLCOM_SHOW_BINLOGS:
+    case SQLCOM_SHOW_OPEN_TABLES:
+    case SQLCOM_SHOW_SLAVE_HOSTS:
+    case SQLCOM_SHOW_BINLOG_EVENTS:
+    case SQLCOM_SHOW_WARNS:
+    case SQLCOM_SHOW_ERRORS:
+    case SQLCOM_SHOW_STORAGE_ENGINES:
+    case SQLCOM_SHOW_PRIVILEGES:
+    case SQLCOM_SHOW_STATUS_PROC:
+    case SQLCOM_SHOW_STATUS_FUNC:
+    case SQLCOM_SHOW_PLUGINS:
+    case SQLCOM_SHOW_EVENTS:
+    case SQLCOM_SHOW_PROFILE:
+    case SQLCOM_SHOW_PROFILES:
+    case SQLCOM_SHOW_RELAYLOG_EVENTS:
+      return true;
+    default:
+      return false;
+  }
+}
+
+int launch_hook_trans_begin(THD *thd, Table_ref *all_tables) {
   DBUG_TRACE;
   LEX *lex = thd->lex;
   enum_sql_command sql_command = lex->sql_command;
@@ -1294,12 +1402,10 @@ int launch_hook_trans_begin(THD *thd, TABLE_LIST *all_tables) {
     return 0;
   }
 
-  bool is_show = ((sql_command_flags[sql_command] & CF_STATUS_COMMAND) &&
-                  (sql_command != SQLCOM_BINLOG_BASE64_EVENT)) ||
-                 (sql_command == SQLCOM_SHOW_RELAYLOG_EVENTS);
+  bool is_show = is_show_status(sql_command);
   bool is_set = (sql_command == SQLCOM_SET_OPTION);
-  bool is_query_block = (sql_command == SQLCOM_SELECT);
-  bool is_do = (sql_command == SQLCOM_DO);
+  bool is_query_block =
+      (sql_command == SQLCOM_SELECT || sql_command == SQLCOM_DO);
   bool is_empty = (sql_command == SQLCOM_EMPTY_QUERY);
   bool is_use = (sql_command == SQLCOM_CHANGE_DB);
   bool is_stop_gr = (sql_command == SQLCOM_STOP_GROUP_REPLICATION);
@@ -1307,8 +1413,8 @@ int launch_hook_trans_begin(THD *thd, TABLE_LIST *all_tables) {
   bool is_reset_persist =
       (sql_command == SQLCOM_RESET && lex->option_type == OPT_PERSIST);
 
-  if ((is_set || is_do || is_show || is_empty || is_use || is_stop_gr ||
-       is_shutdown || is_reset_persist) &&
+  if ((is_set || is_show || is_empty || is_use || is_stop_gr || is_shutdown ||
+       is_reset_persist) &&
       !lex->uses_stored_routines()) {
     return 0;
   }
@@ -1316,7 +1422,7 @@ int launch_hook_trans_begin(THD *thd, TABLE_LIST *all_tables) {
   if (is_query_block) {
     bool is_udf = false;
 
-    // if select is an udf function
+    // If SELECT or DO is a UDF
     Query_block *query_block_elem = lex->unit->first_query_block();
     while (query_block_elem != nullptr) {
       for (Item *item : query_block_elem->visible_fields()) {
@@ -1331,7 +1437,7 @@ int launch_hook_trans_begin(THD *thd, TABLE_LIST *all_tables) {
     }
 
     if (!is_udf && all_tables == nullptr) {
-      // SELECT that don't use tables and isn't a UDF
+      // SELECT or DO that don't use tables and isn't a UDF
       hold_command = false;
     }
 
@@ -1342,7 +1448,7 @@ int launch_hook_trans_begin(THD *thd, TABLE_LIST *all_tables) {
       bool is_sys_db = false;
       bool stop_db_check = false;
 
-      for (TABLE_LIST *table = all_tables; table && !stop_db_check;
+      for (Table_ref *table = all_tables; table && !stop_db_check;
            table = table->next_global) {
         assert(table->db && table->table_name);
 
