@@ -1,16 +1,15 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +27,6 @@
 #include <sys/types.h>
 #include <atomic>
 
-#include "m_string.h"
 #include "my_bitmap.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
@@ -49,6 +47,12 @@ struct mysql_cond_t;
 struct mysql_mutex_t;
 class Rpl_channel_filters;
 
+/*
+  Statistics go to the error log every # of seconds when
+  --log_error_verbosity > 2
+*/
+const long mts_online_stat_period = 60 * 2;
+
 typedef struct struct_slave_connection LEX_SLAVE_CONNECTION;
 
 typedef enum {
@@ -279,7 +283,7 @@ extern bool server_id_supplied;
       rli.data_lock, (relay.reset_logs) THD::LOCK_thd_data,
       relay.LOCK_log, relay.LOCK_index, global_sid_lock->wrlock
 
-    reset_master:
+    reset_binary_logs_and_gtids:
       (binlog.reset_logs) THD::LOCK_thd_data, binlog.LOCK_log,
       binlog.LOCK_index, global_sid_lock->wrlock, LOCK_reset_gtid_table
 
@@ -293,10 +297,9 @@ extern bool server_id_supplied;
 
       [Note: purge_logs contains a known bug: LOCK_index should not be
       taken before LOCK_thd_list.  This implies that, e.g.,
-      purge_source_logs_to_file can deadlock with reset_master.  However,
-      although purge_first_log and reset_slave take locks in reverse
-      order, they cannot deadlock because they both first acquire
-      rli.data_lock.]
+      purge_source_logs_to_file can deadlock with reset_binary_logs_and_gtids.
+  However, although purge_first_log and reset_slave take locks in reverse order,
+  they cannot deadlock because they both first acquire rli.data_lock.]
 
     purge_source_logs_to_file, purge_source_logs_before_date, purge:
       (binlog.purge_logs) binlog.LOCK_index, LOCK_thd_list, thd.linfo.lock
@@ -633,9 +636,6 @@ int connect_to_master(THD *thd, MYSQL *mysql, Master_info *mi, bool reconnect,
 bool net_request_file(NET *net, const char *fname);
 
 extern bool replicate_same_server_id;
-
-extern int disconnect_slave_event_count, abort_slave_event_count;
-
 /* the master variables are defaults read from my.cnf or command line */
 extern uint report_port;
 extern const char *master_info_file;
