@@ -1,16 +1,15 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -73,6 +72,8 @@
 #include "sql/sql_lex.h"
 #include "sql/table.h"
 #include "sql/transaction_info.h"
+#include "string_with_len.h"
+#include "strmake.h"
 #include "thr_mutex.h"
 
 #ifndef NDEBUG
@@ -349,11 +350,15 @@ int Slave_worker::init_worker(Relay_log_info *rli, ulong i) {
   server_version = version_product(rli->slave_version_split);
   bitmap_shifted = 0;
   workers = c_rli->workers;  // shallow copying is sufficient
-  wq_empty_waits = wq_size_waits_cnt = groups_done = events_done = curr_jobs =
-      0;
+  wq_empty_waits = 0;
+  wq_size_waits_cnt = 0;
+  groups_done = 0;
+  events_done = 0;
+  curr_jobs = 0;
   usage_partition = 0;
   end_group_sets_max_dbs = false;
-  gaq_index = last_group_done_index = c_rli->gaq->capacity;  // out of range
+  gaq_index = c_rli->gaq->capacity;              // out of range
+  last_group_done_index = c_rli->gaq->capacity;  // out of range
   last_groups_assigned_index = 0;
   assert(!jobs.inited_queue);
   jobs.avail = 0;
@@ -644,7 +649,6 @@ bool Slave_worker::commit_positions(Log_event *ev, Slave_job_group *ptr_g,
     after a rotation.
   */
   if (ptr_g->group_master_log_name != nullptr) {
-    my_claim(ptr_g->group_master_log_name, /*claim=*/true);
     strmake(group_master_log_name, ptr_g->group_master_log_name,
             sizeof(group_master_log_name) - 1);
     my_free(ptr_g->group_master_log_name);
@@ -653,9 +657,6 @@ bool Slave_worker::commit_positions(Log_event *ev, Slave_job_group *ptr_g,
             sizeof(checkpoint_master_log_name) - 1);
   }
   if (ptr_g->checkpoint_log_name != nullptr) {
-    my_claim(ptr_g->checkpoint_log_name, /*claim=*/true);
-    my_claim(ptr_g->checkpoint_relay_log_name, /*claim=*/true);
-
     strmake(checkpoint_relay_log_name, ptr_g->checkpoint_relay_log_name,
             sizeof(checkpoint_relay_log_name) - 1);
     checkpoint_relay_log_pos = ptr_g->checkpoint_relay_log_pos;
@@ -1603,11 +1604,8 @@ static bool may_have_timestamp(Log_event *ev) {
   bool res = false;
 
   switch (ev->get_type_code()) {
-    case binary_log::QUERY_EVENT:
-      res = true;
-      break;
-
-    case binary_log::GTID_LOG_EVENT:
+    case mysql::binlog::event::QUERY_EVENT:
+    case mysql::binlog::event::GTID_LOG_EVENT:
       res = true;
       break;
 
@@ -1622,7 +1620,7 @@ static int64 get_last_committed(Log_event *ev) {
   int64 res = SEQ_UNINIT;
 
   switch (ev->get_type_code()) {
-    case binary_log::GTID_LOG_EVENT:
+    case mysql::binlog::event::GTID_LOG_EVENT:
       res = static_cast<Gtid_log_event *>(ev)->last_committed;
       break;
 
@@ -1637,7 +1635,7 @@ static int64 get_sequence_number(Log_event *ev) {
   int64 res = SEQ_UNINIT;
 
   switch (ev->get_type_code()) {
-    case binary_log::GTID_LOG_EVENT:
+    case mysql::binlog::event::GTID_LOG_EVENT:
       res = static_cast<Gtid_log_event *>(ev)->sequence_number;
       break;
 
@@ -1786,9 +1784,10 @@ void Slave_worker::report_commit_order_deadlock() {
 
 void Slave_worker::prepare_for_retry(Log_event &event) {
   if (event.get_type_code() ==
-      binary_log::ROWS_QUERY_LOG_EVENT) {  // If a `Rows_query_log_event`, let
-                                           // the event be disposed in the main
-                                           // worker loop.
+      mysql::binlog::event::
+          ROWS_QUERY_LOG_EVENT) {  // If a `Rows_query_log_event`, let
+                                   // the event be disposed in the main
+                                   // worker loop.
     event.worker = this;
     this->rows_query_ev = nullptr;
   }
@@ -1851,10 +1850,10 @@ std::tuple<bool, bool, uint> Slave_worker::check_and_report_end_of_retries(
   return std::make_tuple(false, silent, error);
 }
 
-bool Slave_worker::retry_transaction(my_off_t start_relay_pos,
-                                     const char *start_event_relay_log_name,
-                                     my_off_t end_relay_pos,
-                                     const char *end_event_relay_log_name) {
+bool Slave_worker::retry_transaction(uint start_relay_number,
+                                     my_off_t start_relay_pos,
+                                     uint end_relay_number,
+                                     my_off_t end_relay_pos) {
   THD *thd = info_thd;
   bool silent = false;
 
@@ -1927,36 +1926,37 @@ bool Slave_worker::retry_transaction(my_off_t start_relay_pos,
     clean_retry_context();
     worker_sleep(min<ulong>(trans_retries, MAX_SLAVE_RETRY_PAUSE));
 
-  } while (read_and_apply_events(start_relay_pos, start_event_relay_log_name,
-                                 end_relay_pos, end_event_relay_log_name));
+  } while (read_and_apply_events(start_relay_number, start_relay_pos,
+                                 end_relay_number, end_relay_pos));
   return false;
 }
 
 /**
   Read events from relay logs and apply them.
 
-  @param[in] start_relay_pos The offset of the transaction's first event.
-  @param[in] start_event_relay_log_name The name of the relay log which
+  @param[in] start_relay_number The extension number of the relay log which
                includes the first event of the transaction.
+  @param[in] start_relay_pos The offset of the transaction's first event.
 
-  @param[in] end_relay_pos The offset of the last event it should retry.
-  @param[in] end_event_relay_log_name The name of the relay log which
+  @param[in] end_relay_number The extension number of the relay log which
                includes the last event it should retry.
+  @param[in] end_relay_pos The offset of the last event it should retry.
 
   @return false if succeeds, otherwise returns true.
 */
-bool Slave_worker::read_and_apply_events(my_off_t start_relay_pos,
-                                         const char *start_event_relay_log_name,
-                                         my_off_t end_relay_pos,
-                                         const char *end_event_relay_log_name) {
+bool Slave_worker::read_and_apply_events(uint start_relay_number,
+                                         my_off_t start_relay_pos,
+                                         uint end_relay_number,
+                                         my_off_t end_relay_pos) {
   DBUG_TRACE;
 
   Relay_log_info *rli = c_rli;
   char file_name[FN_REFLEN + 1];
+  uint file_number = start_relay_number;
   bool arrive_end = false;
   Relaylog_file_reader relaylog_file_reader(opt_replica_sql_verify_checksum);
 
-  strcpy(file_name, start_event_relay_log_name);
+  relay_log_number_to_name(start_relay_number, file_name);
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
   PSI_thread *thread = thd_get_psi(rli->info_thd);
@@ -1980,7 +1980,7 @@ bool Slave_worker::read_and_apply_events(my_off_t start_relay_pos,
 
     /* If it is the last event, then set arrive_end as true */
     arrive_end = (relaylog_file_reader.position() == end_relay_pos &&
-                  !(strcmp(file_name, end_event_relay_log_name)));
+                  file_number == end_relay_number);
 
     ev = relaylog_file_reader.read_event_object();
     if (ev != nullptr) {
@@ -2035,6 +2035,8 @@ bool Slave_worker::read_and_apply_events(my_off_t start_relay_pos,
         return true;
       }
 
+      file_number = relay_log_name_to_number(file_name);
+
       relaylog_file_reader.close();
       start_relay_pos = BIN_LOG_HEADER_SIZE;
     }
@@ -2098,8 +2100,6 @@ bool append_item_to_jobs(slave_job_item *job_item, Slave_worker *worker,
   mysql_mutex_lock(&rli->pending_jobs_lock);
   new_pend_size = rli->mts_pending_jobs_size + ev_size;
   bool big_event = (ev_size > rli->mts_pending_jobs_size_max);
-  Slave_job_group *ptr_g =
-      rli->gaq->get_job_group(rli->gaq->assigned_group_index);
   /*
     C waits basing on *data* sizes in the queues.
     If it is a big event (event size is greater than
@@ -2124,9 +2124,15 @@ bool append_item_to_jobs(slave_job_item *job_item, Slave_worker *worker,
     mysql_mutex_unlock(&rli->pending_jobs_lock);
     thd->EXIT_COND(&old_stage);
     if (thd->killed) return true;
-    if (rli->wq_size_waits_cnt % 10 == 1)
-      LogErr(INFORMATION_LEVEL, ER_RPL_MTA_REPLICA_COORDINATOR_HAS_WAITED,
-             rli->wq_size_waits_cnt, ev_size);
+    if (rli->wq_size_waits_cnt % 10 == 1) {
+      time_t my_now = time(nullptr);
+      if ((my_now - rli->mta_coordinator_has_waited_stat) >=
+          mts_online_stat_period) {
+        LogErr(INFORMATION_LEVEL, ER_RPL_MTA_REPLICA_COORDINATOR_HAS_WAITED,
+               rli->wq_size_waits_cnt, ev_size);
+        rli->mta_coordinator_has_waited_stat = my_now;
+      }
+    }
     mysql_mutex_lock(&rli->pending_jobs_lock);
 
     new_pend_size = rli->mts_pending_jobs_size + ev_size;
@@ -2166,16 +2172,6 @@ bool append_item_to_jobs(slave_job_item *job_item, Slave_worker *worker,
     rli->mts_wq_no_underrun_cnt++;
   }
 
-  // unclaim ownership of the event log memory
-  job_item->data->claim_memory_ownership(/*claim=*/false);
-  if (worker->checkpoint_notified) {
-    my_claim(ptr_g->checkpoint_log_name, /*claim=*/false);
-    my_claim(ptr_g->checkpoint_relay_log_name, /*claim=*/false);
-  }
-  if (worker->master_log_change_notified) {
-    my_claim(ptr_g->group_master_log_name, /*claim=*/false);
-  }
-
   mysql_mutex_lock(&worker->jobs_lock);
 
   // possible WQ overfill
@@ -2205,16 +2201,6 @@ bool append_item_to_jobs(slave_job_item *job_item, Slave_worker *worker,
     rli->pending_jobs--;  // roll back of the prev incr
     rli->mts_pending_jobs_size -= ev_size;
     mysql_mutex_unlock(&rli->pending_jobs_lock);
-
-    // claim back ownership of the event log memory
-    job_item->data->claim_memory_ownership(/*claim=*/true);
-    if (worker->checkpoint_notified) {
-      my_claim(ptr_g->checkpoint_log_name, /*claim=*/true);
-      my_claim(ptr_g->checkpoint_relay_log_name, /*claim=*/true);
-    }
-    if (worker->master_log_change_notified) {
-      my_claim(ptr_g->group_master_log_name, /*claim=*/true);
-    }
   }
 
   return (Slave_jobs_queue::error_result != ret) ? false : true;
@@ -2423,23 +2409,23 @@ void report_error_to_coordinator(Slave_worker *worker) {
          returns an error code.
  */
 int slave_worker_exec_job_group(Slave_worker *worker, Relay_log_info *rli) {
-  struct slave_job_item item = {nullptr, 0, {'\0'}};
+  struct slave_job_item item = {nullptr, 0, 0};
   struct slave_job_item *job_item = &item;
   THD *thd = worker->info_thd;
   bool seen_gtid = false;
   bool seen_begin = false;
   int error = 0;
   Log_event *ev = nullptr;
+  uint start_relay_number;
   my_off_t start_relay_pos;
-  char start_event_relay_log_name[FN_REFLEN + 1];
 
   DBUG_TRACE;
 
   if (unlikely(worker->trans_retries > 0)) worker->trans_retries = 0;
 
   job_item = pop_jobs_item(worker, job_item);
+  start_relay_number = job_item->relay_number;
   start_relay_pos = job_item->relay_pos;
-  strcpy(start_event_relay_log_name, job_item->event_relay_log_name);
 
   PSI_thread *thread = thd_get_psi(thd);
 
@@ -2468,10 +2454,6 @@ int slave_worker_exec_job_group(Slave_worker *worker, Relay_log_info *rli) {
 
     ev = job_item->data;
     assert(ev != nullptr);
-
-    // claim ownership of the event log memory
-    ev->claim_memory_ownership(/*claim=*/true);
-
     DBUG_PRINT("info", ("W_%lu <- job item: %p data: %p thd: %p", worker->id,
                         job_item, ev, thd));
     /*
@@ -2508,9 +2490,9 @@ int slave_worker_exec_job_group(Slave_worker *worker, Relay_log_info *rli) {
         diff_timespec(&worker->ts_exec[1], &worker->ts_exec[0]);
     if (error || worker->found_commit_order_deadlock()) {
       worker->prepare_for_retry(*ev);
-      error = worker->retry_transaction(
-          start_relay_pos, start_event_relay_log_name, job_item->relay_pos,
-          job_item->event_relay_log_name);
+      error = worker->retry_transaction(start_relay_number, start_relay_pos,
+                                        job_item->relay_number,
+                                        job_item->relay_pos);
       if (error) goto err;
     }
     /*
@@ -2522,13 +2504,14 @@ int slave_worker_exec_job_group(Slave_worker *worker, Relay_log_info *rli) {
       with the final disjunct.
     */
     assert(seen_begin || is_gtid_event(ev) ||
-           ev->get_type_code() == binary_log::QUERY_EVENT ||
+           ev->get_type_code() == mysql::binlog::event::QUERY_EVENT ||
            is_mts_db_partitioned(rli) || worker->id == 0 || seen_gtid);
 
-    if (ev->ends_group() || (!seen_begin && !is_gtid_event(ev) &&
-                             (ev->get_type_code() == binary_log::QUERY_EVENT ||
-                              /* break through by LC only in GTID off */
-                              (!seen_gtid && !is_mts_db_partitioned(rli)))))
+    if (ev->ends_group() ||
+        (!seen_begin && !is_gtid_event(ev) &&
+         (ev->get_type_code() == mysql::binlog::event::QUERY_EVENT ||
+          /* break through by LC only in GTID off */
+          (!seen_gtid && !is_mts_db_partitioned(rli)))))
       break;
 
     remove_item_from_jobs(job_item, worker, rli);
@@ -2555,7 +2538,7 @@ int slave_worker_exec_job_group(Slave_worker *worker, Relay_log_info *rli) {
       assert(opt_debug_sync_timeout > 0);
       assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
     };);
-    if (ev->get_type_code() == binary_log::QUERY_EVENT &&
+    if (ev->get_type_code() == mysql::binlog::event::QUERY_EVENT &&
         ((Query_log_event *)ev)->rollback_injected_by_coord) {
       /*
         If this was a rollback event injected by the coordinator because of a
