@@ -1,17 +1,16 @@
 /*
-   Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is designed to work with certain software (including
+   This program is also distributed with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have either included with
-   the program or referenced in the documentation.
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,7 +41,6 @@
 #include <vector>
 
 #include "lex_string.h"
-#include "m_ctype.h"
 #include "m_string.h"
 #include "my_command.h"
 #include "my_dbug.h"
@@ -57,9 +55,12 @@
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/int2str.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "mysys_err.h"  // EE_*
+#include "nulls.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"  // SELECT_ACL
 #include "sql/auth/sql_security_ctx.h"
@@ -99,6 +100,9 @@
 #include "sql/thd_raii.h"
 #include "sql/transaction.h"  // trans_rollback_stmt
 #include "sql_string.h"
+#include "string_with_len.h"
+#include "strmake.h"
+#include "strxmov.h"
 #include "typelib.h"
 
 /*
@@ -149,7 +153,7 @@ bool get_default_db_collation(THD *thd, const char *db_name,
                               const CHARSET_INFO **collation) {
   // We must make sure the schema is released and unlocked in the right order.
   dd::Schema_MDL_locker mdl_handler(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *sch_obj = nullptr;
 
   if (mdl_handler.ensure_locked(db_name) ||
@@ -234,7 +238,7 @@ bool check_schema_readonly(THD *thd, const char *schema_name,
   }
 
   dd::Schema_MDL_locker mdl_handler(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *sch_obj = nullptr;
 
   if (mdl_handler.ensure_locked(schema_name) ||
@@ -262,7 +266,7 @@ bool check_schema_readonly(THD *thd, const char *schema_name,
 
 static bool write_db_cmd_to_binlog(THD *thd, const char *db, bool trx_cache) {
   if (mysql_bin_log.is_open()) {
-    int errcode = query_error_code(thd, true);
+    const int errcode = query_error_code(thd, true);
     Query_log_event qinfo(thd, thd->query().str, thd->query().length, trx_cache,
                           false, /* suppress_use */ true, errcode);
     /*
@@ -329,7 +333,7 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
     Use Auto_releaser to keep uncommitted object for database until
     trans_commit() call.
   */
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   // Reject creation of the system schema except for system threads.
   if (!thd->is_dd_system_thread() &&
@@ -384,7 +388,8 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   }
 
   bool store_in_dd = true;
-  bool if_not_exists = (create_info->options & HA_LEX_CREATE_IF_NOT_EXISTS);
+  const bool if_not_exists =
+      (create_info->options & HA_LEX_CREATE_IF_NOT_EXISTS);
   if (existing_schema != nullptr) {
     if (if_not_exists == false) {
       my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
@@ -399,8 +404,8 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   /* Check directory */
   char path[FN_REFLEN + 16];
   bool was_truncated;
-  size_t path_len = build_table_filename(path, sizeof(path) - 1, db, "", "", 0,
-                                         &was_truncated);
+  const size_t path_len = build_table_filename(path, sizeof(path) - 1, db, "",
+                                               "", 0, &was_truncated);
   if (was_truncated) {
     my_error(ER_IDENT_CAUSES_TOO_LONG_PATH, MYF(0), sizeof(path) - 1, path);
     return true;
@@ -413,7 +418,7 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   // the physical representation of the schema is not re-created since it
   // already exists.
   MY_STAT stat_info;
-  bool schema_dir_exists =
+  const bool schema_dir_exists =
       (mysql_file_stat(key_file_misc, path, &stat_info, MYF(0)) != nullptr);
   if (thd->is_dd_system_thread() &&
       (!opt_initialize || dd::upgrade_57::in_progress()) &&
@@ -531,7 +536,7 @@ bool mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
 
   if (lock_schema_name(thd, db)) return true;
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   dd::Schema *schema = nullptr;
   if (thd->dd_client()->acquire_for_modification(db, &schema)) return true;
 
@@ -715,7 +720,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
 
   DBUG_TRACE;
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   // Reject dropping the system schema except for system threads.
   if (!thd->is_dd_system_thread() &&
@@ -837,7 +842,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
 
       ha_drop_database(path);
       thd->clear_error(); /* @todo Do not ignore errors */
-      Disable_binlog_guard binlog_guard(thd);
+      const Disable_binlog_guard binlog_guard(thd);
       error = Events::drop_schema_events(thd, *schema);
       error = (error || sp_drop_db_routines(thd, *schema));
     }
@@ -1360,7 +1365,7 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
 
   // We must make sure the schema is released and unlocked in the right order.
   dd::Schema_MDL_locker mdl_handler(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *schema = nullptr;
 
   DBUG_TRACE;
@@ -1431,14 +1436,14 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
   DBUG_PRINT("info", ("Use database: %s", new_db_file_name.str));
 
   if (sctx->get_active_roles()->size() == 0) {
-    db_access =
-        sctx->check_access(DB_OP_ACLS, new_db_file_name.str)
-            ? DB_OP_ACLS
-            : acl_get(thd, sctx->host().str, sctx->ip().str,
-                      sctx->priv_user().str, new_db_file_name.str, false) |
-                  sctx->master_access(new_db_file_name.str);
+    db_access = sctx->check_access(DB_OP_ACLS, new_db_file_name.str)
+                    ? DB_OP_ACLS
+                    : sctx->check_db_level_access(thd, new_db_file_name.str,
+                                                  new_db_file_name.length) |
+                          sctx->master_access(new_db_file_name.str);
   } else {
-    db_access = sctx->db_acl(new_db_file_name_cstr) |
+    db_access = sctx->check_db_level_access(thd, new_db_file_name.str,
+                                            new_db_file_name.length) |
                 sctx->master_access(new_db_file_name.str);
   }
 
