@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_11858416_1687334106_05_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_11858416_1687334106_05_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_12964073_1697111053_03_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_12964073_1697111053_03_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef MY_SQL_PARSER_DEBUG
 # if defined YYDEBUG
@@ -850,11 +850,15 @@ extern int my_sql_parser_debug;
     GENERATE_SYM = 1203,
     DOLLAR_QUOTED_STRING_SYM = 1204,
     PARSE_TREE_SYM = 1205,
-    KEYWORD_USED_AS_IDENT = 1207,
-    KEYWORD_USED_AS_KEYWORD = 1208,
-    CONDITIONLESS_JOIN = 1209,
-    SUBQUERY_AS_EXPR = 1210,
-    EMPTY_FROM_CLAUSE = 1211
+    LOG_SYM = 1206,
+    GTIDS_SYM = 1207,
+    PARALLEL_SYM = 1208,
+    S3_SYM = 1209,
+    KEYWORD_USED_AS_IDENT = 1211,
+    KEYWORD_USED_AS_KEYWORD = 1212,
+    CONDITIONLESS_JOIN = 1213,
+    SUBQUERY_AS_EXPR = 1214,
+    EMPTY_FROM_CLAUSE = 1215
   };
 #endif
 /* Tokens.  */
@@ -1655,11 +1659,15 @@ extern int my_sql_parser_debug;
 #define GENERATE_SYM 1203
 #define DOLLAR_QUOTED_STRING_SYM 1204
 #define PARSE_TREE_SYM 1205
-#define KEYWORD_USED_AS_IDENT 1207
-#define KEYWORD_USED_AS_KEYWORD 1208
-#define CONDITIONLESS_JOIN 1209
-#define SUBQUERY_AS_EXPR 1210
-#define EMPTY_FROM_CLAUSE 1211
+#define LOG_SYM 1206
+#define GTIDS_SYM 1207
+#define PARALLEL_SYM 1208
+#define S3_SYM 1209
+#define KEYWORD_USED_AS_IDENT 1211
+#define KEYWORD_USED_AS_KEYWORD 1212
+#define CONDITIONLESS_JOIN 1213
+#define SUBQUERY_AS_EXPR 1214
+#define EMPTY_FROM_CLAUSE 1215
 
 /* Value type.  */
 
@@ -1681,4 +1689,4 @@ struct MY_SQL_PARSER_LTYPE
 
 int my_sql_parser_parse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_11858416_1687334106_05_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_12964073_1697111053_03_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
