@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2021, Oracle and/or its affiliates.
+Copyright (c) 1995, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -40,16 +40,16 @@ this program; if not, write to the Free Software Foundation, Inc.,
 buffer buf_pool if it is not already there, in which case does nothing.
 Sets the io_fix flag and sets an exclusive lock on the buffer frame. The
 flag is cleared and the x-lock released by an i/o-handler thread.
-@param[out]	err		DB_SUCCESS or DB_TABLESPACE_DELETED
+@param[out]     err             DB_SUCCESS or DB_TABLESPACE_DELETED
                                 if we are trying to read from a non-existent
                                 tablespace or a tablespace which is just now
                                 being dropped
-@param[in]	sync		whether synchronous aio is desired
-@param[in]	type		Request type
-@param[in]	mode		BUF_READ_IBUF_PAGES_ONLY, ...
-@param[in]	page_id		page id
-@param[in]	page_size	page size
-@param[in]	unzip		true=request uncompressed page
+@param[in]      sync            whether synchronous aio is desired
+@param[in]      type            Request type
+@param[in]      mode            BUF_READ_IBUF_PAGES_ONLY, ...
+@param[in]      page_id         page id
+@param[in]      page_size       page size
+@param[in]      unzip           true=request uncompressed page
 @return 1 if a read request was queued, 0 if the page already resided in
 buf_pool, or if the page is in the doublewrite buffer blocks in which case it
 is never read into the pool, or if the tablespace does not exist or is being
@@ -62,8 +62,8 @@ ulint buf_read_page_low(dberr_t *err, bool sync, ulint type, ulint mode,
 buffer buf_pool if it is not already there. Sets the io_fix flag and sets
 an exclusive lock on the buffer frame. The flag is cleared and the x-lock
 released by the i/o-handler thread.
-@param[in]	page_id		page id
-@param[in]	page_size	page size
+@param[in]      page_id         page id
+@param[in]      page_size       page size
 @return true if page has been read in, false in case of failure */
 bool buf_read_page(const page_id_t &page_id, const page_size_t &page_size);
 
@@ -71,9 +71,9 @@ bool buf_read_page(const page_id_t &page_id, const page_size_t &page_size);
 buffer buf_pool if it is not already there. Sets the io_fix flag and sets
 an exclusive lock on the buffer frame. The flag is cleared and the x-lock
 released by the i/o-handler thread.
-@param[in]	page_id		page id
-@param[in]	page_size	page size
-@param[in]	sync		true if synchronous aio is desired
+@param[in]      page_id         page id
+@param[in]      page_size       page size
+@param[in]      sync            true if synchronous aio is desired
 @return true if page has been read in, false in case of failure */
 bool buf_read_page_background(const page_id_t &page_id,
                               const page_size_t &page_size, bool sync);
@@ -87,10 +87,10 @@ end up waiting for these latches! NOTE 2: the calling thread must want
 access to the page given: this rule is set to prevent unintended read-aheads
 performed by ibuf routines, a situation which could result in a deadlock if
 the OS does not support asynchronous i/o.
-@param[in]	page_id		page id of a page which the current thread
+@param[in]      page_id         page id of a page which the current thread
 wants to access
-@param[in]	page_size	page size
-@param[in]	inside_ibuf	TRUE if we are inside ibuf routine
+@param[in]      page_size       page size
+@param[in]      inside_ibuf     true if we are inside ibuf routine
 @return number of page read requests issued; NOTE that if we read ibuf
 pages, it may happen that the page at the given page number does not
 get read even if we return a positive value! */
@@ -119,9 +119,9 @@ latches!
 NOTE 3: the calling thread must want access to the page given: this rule is
 set to prevent unintended read-aheads performed by ibuf routines, a situation
 which could result in a deadlock if the OS does not support asynchronous io.
-@param[in]	page_id		page id; see NOTE 3 above
-@param[in]	page_size	page size
-@param[in]	inside_ibuf	TRUE if we are inside ibuf routine
+@param[in]      page_id         page id; see NOTE 3 above
+@param[in]      page_size       page size
+@param[in]      inside_ibuf     true if we are inside ibuf routine
 @return number of page read requests issued */
 ulint buf_read_ahead_linear(const page_id_t &page_id,
                             const page_size_t &page_size, bool inside_ibuf);
@@ -139,27 +139,21 @@ a read-ahead function.
 void buf_read_ibuf_merge_pages(bool sync, const space_id_t *space_ids,
                                const page_no_t *page_nos, ulint n_stored);
 
-/** Issues read requests for pages which recovery wants to read in.
-@param[in]	sync		true if the caller wants this function to wait
-for the highest address page to get read in, before this function returns
-@param[in]	space_id	tablespace id
-@param[in]	page_nos	array of page numbers to read, with the
+/** Issues asynchronous read requests for pages which recovery wants to read in.
+@param[in]      space_id        tablespace id
+@param[in]      page_nos        array of page numbers to read, with the
 highest page number the last in the array
-@param[in]	n_stored	number of page numbers in the array */
+@param[in]      n_stored        number of page numbers in the array */
 
-void buf_read_recv_pages(bool sync, space_id_t space_id,
-                         const page_no_t *page_nos, ulint n_stored);
-
-/** The size in pages of the area which the read-ahead algorithms read if
-invoked */
-#define BUF_READ_AHEAD_AREA(b) ((b)->read_ahead_area)
+void buf_read_recv_pages(space_id_t space_id, const page_no_t *page_nos,
+                         ulint n_stored);
 
 /** @name Modes used in read-ahead
 @{ */
 /** read only pages belonging to the insert buffer tree */
-#define BUF_READ_IBUF_PAGES_ONLY 131
+constexpr uint32_t BUF_READ_IBUF_PAGES_ONLY = 131;
 /** read any page */
-#define BUF_READ_ANY_PAGE 132
+constexpr uint32_t BUF_READ_ANY_PAGE = 132;
 /** @} */
 
 #endif
