@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,6 +22,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include "util/require.h"
 #include <NDBT_Test.hpp>
 #include <NDBT_ReturnCodes.h>
 #include <HugoTransactions.hpp>
@@ -1166,7 +1167,7 @@ NegativeTest::runSetBoundTest() const
       return NDBT_FAILED;
     }
 
-    // Set correct string lengh.
+    // Set correct string length.
     boundRow.len = nt2StrLen;
     bound.range_no = 1;
     if (query->setBound(ordIdxRecord, &bound) == 0 ||
