@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -22,6 +22,8 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include "util/require.h"
+#include <cstring>
 #include <HugoOperations.hpp>
 
 #undef NDB_ERR
@@ -68,6 +70,26 @@ int HugoOperations::startTransaction(Ndb* pNdb,
   return NDBT_OK;
 }
 
+int HugoOperations::startTransaction(Ndb* pNdb,
+                                     int recordNo)
+{
+  if (pTrans != NULL)
+  {
+    ndbout << "HugoOperations::startTransaction, pTrans != NULL" << endl;
+    return NDBT_FAILED;
+  }
+  HugoCalculator::KeyParts kp;
+  calc.setKeyParts(&kp, recordNo);
+  pTrans = pNdb->startTransaction(&tab, kp.ptrs);
+  if (pTrans == NULL) {
+    const NdbError err = pNdb->getNdbError();
+    NDB_ERR(err);
+    setNdbError(err);
+    return NDBT_FAILED;
+  }
+  return NDBT_OK;
+}
+
 int HugoOperations::setTransaction(NdbTransaction* new_trans, bool not_null_ok){
   
   if (pTrans != NULL && !not_null_ok){
@@ -664,7 +686,7 @@ int HugoOperations::pkRefreshRecord(Ndb* pNdb,
   opts.optionsPresent = NdbOperation::OperationOptions::OO_ANYVALUE;
   for(int r=0; r < numRecords; r++)
   {
-    bzero(buffer, sizeof(buffer));
+    std::memset(buffer, 0, sizeof(buffer));
     if (calc.equalForRow((Uint8*)buffer, record, r + recordNo))
     {
       g_err << __LINE__ << " equal for row failed" << endl;
@@ -961,7 +983,7 @@ int HugoOperations::equalForAttr(NdbOperation* pOp,
   
   int len = attr->getSizeInBytes();
   char buf[NDB_MAX_TUPLE_SIZE];
-  memset(buf, 0, sizeof(buf));
+  std::memset(buf, 0, sizeof(buf));
   Uint32 real_len;
   const char * value = calc.calcValue(rowId, attrId, 0, buf, len, &real_len);
   return pOp->equal( attr->getName(), value, real_len);
@@ -977,7 +999,7 @@ int HugoOperations::setValueForAttr(NdbOperation* pOp,
   {
     int len = attr->getSizeInBytes();
     char buf[NDB_MAX_TUPLE_SIZE];
-    memset(buf, 0, sizeof(buf));
+    std::memset(buf, 0, sizeof(buf));
     Uint32 real_len;
     const char * value = calc.calcValue(rowId, attrId,
                                         updateId, buf, len, &real_len);
