@@ -1,16 +1,15 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is designed to work with certain software (including
+  This program is also distributed with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have either included with
-  the program or referenced in the documentation.
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,7 +30,6 @@
 #include <string.h>
 #include <algorithm>
 
-#include "m_ctype.h"
 #include "m_string.h"
 #include "my_bitmap.h"
 #include "my_byteorder.h"
@@ -42,6 +40,7 @@
 #include "myisampack.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/strings/m_ctype.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/current_thd.h"
 #include "sql/field.h"
@@ -134,6 +133,8 @@
 #include "storage/perfschema/table_setup_actors.h"
 #include "storage/perfschema/table_setup_consumers.h"
 #include "storage/perfschema/table_setup_instruments.h"
+#include "storage/perfschema/table_setup_meters.h"
+#include "storage/perfschema/table_setup_metrics.h"
 #include "storage/perfschema/table_setup_objects.h"
 #include "storage/perfschema/table_setup_threads.h"
 #include "storage/perfschema/table_socket_instances.h"
@@ -503,6 +504,8 @@ static PFS_engine_table_share *all_shares[] = {
     &table_setup_actors::m_share,
     &table_setup_consumers::m_share,
     &table_setup_instruments::m_share,
+    &table_setup_meters::m_share,
+    &table_setup_metrics::m_share,
     &table_setup_objects::m_share,
     &table_setup_threads::m_share,
     &table_tiws_by_index_usage::m_share,
@@ -902,8 +905,7 @@ class PFS_internal_schema_access : public ACL_internal_schema_access {
 
   ~PFS_internal_schema_access() override = default;
 
-  ACL_internal_access_result check(Access_bitmask want_access,
-                                   Access_bitmask *save_priv,
+  ACL_internal_access_result check(ulong want_access, ulong *save_priv,
                                    bool any_combination_will_do) const override;
 
   const ACL_internal_table_access *lookup(const char *name) const override;
@@ -943,9 +945,10 @@ static bool allow_drop_schema_privilege() {
           (thd->lex->sql_command == SQLCOM_DROP_TABLE));
 }
 
-ACL_internal_access_result PFS_internal_schema_access::check(
-    Access_bitmask want_access, Access_bitmask *, bool) const {
-  constexpr Access_bitmask always_forbidden =
+ACL_internal_access_result PFS_internal_schema_access::check(ulong want_access,
+                                                             ulong *,
+                                                             bool) const {
+  const ulong always_forbidden =
       CREATE_ACL | REFERENCES_ACL | INDEX_ACL | ALTER_ACL | CREATE_TMP_ACL |
       EXECUTE_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | CREATE_PROC_ACL |
       ALTER_PROC_ACL | EVENT_ACL | TRIGGER_ACL;
@@ -1026,18 +1029,18 @@ static bool allow_drop_table_privilege() {
 PFS_readonly_acl pfs_readonly_acl;
 
 ACL_internal_access_result PFS_readonly_acl::check(
-    Access_bitmask want_access, Access_bitmask *granted_access,
+    ulong want_access, ulong *granted_access,
     bool any_combination_will_do) const {
-  constexpr Access_bitmask always_forbidden =
-      INSERT_ACL | UPDATE_ACL | DELETE_ACL | CREATE_ACL | DROP_ACL |
-      REFERENCES_ACL | INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
-      TRIGGER_ACL | LOCK_TABLES_ACL;
+  const ulong always_forbidden = INSERT_ACL | UPDATE_ACL | DELETE_ACL |
+                                 CREATE_ACL | DROP_ACL | REFERENCES_ACL |
+                                 INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL |
+                                 SHOW_VIEW_ACL | TRIGGER_ACL | LOCK_TABLES_ACL;
 
-  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  const Access_bitmask want_forbidden = want_access & always_forbidden;
+  const ulong want_forbidden = want_access & always_forbidden;
 
-  const Access_bitmask want_allowable = want_access & can_be_allowed;
+  const ulong want_allowable = want_access & can_be_allowed;
 
   if (any_combination_will_do) {
     if (want_allowable != 0) {
@@ -1058,8 +1061,7 @@ ACL_internal_access_result PFS_readonly_acl::check(
 PFS_readonly_world_acl pfs_readonly_world_acl;
 
 ACL_internal_access_result PFS_readonly_world_acl::check(
-    Access_bitmask want_access, Access_bitmask *save_priv,
-    bool any_combination_will_do) const {
+    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
   ACL_internal_access_result res =
       PFS_readonly_acl::check(want_access, save_priv, any_combination_will_do);
   if (res == ACL_INTERNAL_ACCESS_CHECK_GRANT) {
@@ -1071,8 +1073,7 @@ ACL_internal_access_result PFS_readonly_world_acl::check(
 PFS_readonly_processlist_acl pfs_readonly_processlist_acl;
 
 ACL_internal_access_result PFS_readonly_processlist_acl::check(
-    Access_bitmask want_access, Access_bitmask *save_priv,
-    bool any_combination_will_do) const {
+    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
   const ACL_internal_access_result res =
       PFS_readonly_acl::check(want_access, save_priv, any_combination_will_do);
 
@@ -1085,18 +1086,18 @@ ACL_internal_access_result PFS_readonly_processlist_acl::check(
 PFS_truncatable_acl pfs_truncatable_acl;
 
 ACL_internal_access_result PFS_truncatable_acl::check(
-    Access_bitmask want_access, Access_bitmask *granted_access,
+    ulong want_access, ulong *granted_access,
     bool any_combination_will_do) const {
-  constexpr Access_bitmask always_forbidden =
-      INSERT_ACL | UPDATE_ACL | DELETE_ACL | CREATE_ACL | REFERENCES_ACL |
-      INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL |
-      LOCK_TABLES_ACL;
+  const ulong always_forbidden = INSERT_ACL | UPDATE_ACL | DELETE_ACL |
+                                 CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
+                                 ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
+                                 TRIGGER_ACL | LOCK_TABLES_ACL;
 
-  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  Access_bitmask want_allowable = want_access & can_be_allowed;
+  ulong want_allowable = want_access & can_be_allowed;
 
-  Access_bitmask want_forbidden = want_access & always_forbidden;
+  ulong want_forbidden = want_access & always_forbidden;
 
   if (want_access & DROP_ACL) {
     if (!allow_drop_table_privilege()) {
@@ -1124,8 +1125,7 @@ ACL_internal_access_result PFS_truncatable_acl::check(
 PFS_truncatable_world_acl pfs_truncatable_world_acl;
 
 ACL_internal_access_result PFS_truncatable_world_acl::check(
-    Access_bitmask want_access, Access_bitmask *save_priv,
-    bool any_combination_will_do) const {
+    ulong want_access, ulong *save_priv, bool any_combination_will_do) const {
   ACL_internal_access_result res = PFS_truncatable_acl::check(
       want_access, save_priv, any_combination_will_do);
   if (res == ACL_INTERNAL_ACCESS_CHECK_GRANT) {
@@ -1137,17 +1137,17 @@ ACL_internal_access_result PFS_truncatable_world_acl::check(
 PFS_updatable_acl pfs_updatable_acl;
 
 ACL_internal_access_result PFS_updatable_acl::check(
-    Access_bitmask want_access, Access_bitmask *granted_access,
+    ulong want_access, ulong *granted_access,
     bool any_combination_will_do) const {
-  constexpr Access_bitmask always_forbidden =
+  const ulong always_forbidden =
       INSERT_ACL | DELETE_ACL | CREATE_ACL | DROP_ACL | REFERENCES_ACL |
       INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL | TRIGGER_ACL;
 
-  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  const Access_bitmask want_forbidden = want_access & always_forbidden;
+  const ulong want_forbidden = want_access & always_forbidden;
 
-  const Access_bitmask want_allowable = want_access & can_be_allowed;
+  const ulong want_allowable = want_access & can_be_allowed;
 
   if (any_combination_will_do) {
     if (want_allowable != 0) {
@@ -1168,17 +1168,17 @@ ACL_internal_access_result PFS_updatable_acl::check(
 PFS_editable_acl pfs_editable_acl;
 
 ACL_internal_access_result PFS_editable_acl::check(
-    Access_bitmask want_access, Access_bitmask *granted_access,
+    ulong want_access, ulong *granted_access,
     bool any_combination_will_do) const {
-  constexpr Access_bitmask always_forbidden =
-      CREATE_ACL | REFERENCES_ACL | INDEX_ACL | ALTER_ACL | CREATE_VIEW_ACL |
-      SHOW_VIEW_ACL | TRIGGER_ACL;
+  const ulong always_forbidden = CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
+                                 ALTER_ACL | CREATE_VIEW_ACL | SHOW_VIEW_ACL |
+                                 TRIGGER_ACL;
 
-  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  Access_bitmask want_forbidden = want_access & always_forbidden;
+  ulong want_forbidden = want_access & always_forbidden;
 
-  Access_bitmask want_allowable = want_access & can_be_allowed;
+  ulong want_allowable = want_access & can_be_allowed;
 
   if (want_access & DROP_ACL) {
     if (!allow_drop_table_privilege()) {
@@ -1206,7 +1206,7 @@ ACL_internal_access_result PFS_editable_acl::check(
 PFS_unknown_acl pfs_unknown_acl;
 
 ACL_internal_access_result PFS_unknown_acl::check(
-    Access_bitmask want_access, Access_bitmask *granted_access,
+    ulong want_access, ulong *granted_access,
     bool any_combination_will_do) const {
   /*
     Only enforce ACL_INTERNAL_ACCESS_DENIED
@@ -1214,15 +1214,14 @@ ACL_internal_access_result PFS_unknown_acl::check(
     in the performance schema,
     relax error messages otherwise.
   */
-  constexpr Access_bitmask always_forbidden = CREATE_ACL | REFERENCES_ACL |
-                                              INDEX_ACL | ALTER_ACL |
-                                              CREATE_VIEW_ACL | TRIGGER_ACL;
+  const ulong always_forbidden = CREATE_ACL | REFERENCES_ACL | INDEX_ACL |
+                                 ALTER_ACL | CREATE_VIEW_ACL | TRIGGER_ACL;
 
-  constexpr Access_bitmask can_be_allowed = TABLE_ACLS & (~always_forbidden);
+  const ulong can_be_allowed = TABLE_ACLS & (~always_forbidden);
 
-  const Access_bitmask want_forbidden = want_access & always_forbidden;
+  const ulong want_forbidden = want_access & always_forbidden;
 
-  const Access_bitmask want_allowable = want_access & can_be_allowed;
+  const ulong want_allowable = want_access & can_be_allowed;
 
   if (any_combination_will_do) {
     if (want_allowable != 0) {
