@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,11 +24,13 @@
 #include <sys/types.h>
 
 #include "my_inttypes.h"
+#include "mysql/strings/m_ctype.h"
+#include "sql-common/my_decimal.h"
 #include "sql/field.h"
-#include "sql/my_decimal.h"
 #include "sql/protocol.h"
 #include "sql/sql_class.h"
 #include "sql/sql_time.h"
+#include "strings/m_ctype_internals.h"
 #include "unittest/gunit/fake_table.h"
 #include "unittest/gunit/mysys_util.h"
 #include "unittest/gunit/test_utils.h"
@@ -98,7 +100,7 @@ class Mock_protocol : public Protocol {
   ulong get_client_capabilities() override { return 0; }
   bool has_client_capability(unsigned long) override { return false; }
   void end_partial_result_set() override {}
-  int shutdown(bool server_shutdown MY_ATTRIBUTE((unused)) = false) override {
+  int shutdown(bool server_shutdown [[maybe_unused]] = false) override {
     return 0;
   }
   SSL_handle get_ssl() { return nullptr; }
@@ -291,10 +293,10 @@ TEST_F(FieldTest, FieldTimef) {
 
   Mock_table m_table(thd());
   f->table = &m_table;
-  struct timeval tv;
+  my_timeval tv;
   int warnings = 0;
   EXPECT_FALSE(f->get_timestamp(&tv, &warnings));
-  EXPECT_EQ(123400, tv.tv_usec);
+  EXPECT_EQ(123400, tv.m_tv_usec);
 
   destroy(field);
 }
@@ -414,7 +416,7 @@ TEST_F(FieldTest, CopyFieldSet) {
   f_from->set_field_ptr(&from_fieldval);
 
   Copy_field *cf = new (thd()->mem_root) Copy_field;
-  cf->set(f_to, f_from, false);
+  cf->set(f_to, f_from);
   cf->invoke_do_copy();
 
   // Copy_field DTOR is not invoked in all contexts, so we may leak memory.
@@ -505,7 +507,8 @@ class Mock_charset : public CHARSET_INFO {
  public:
   mutable bool strnxfrm_called;
   Mock_charset() : strnxfrm_called(false) {
-    cset = &my_charset_8bit_handler;
+    CHARSET_INFO *ci = get_charset_by_name("latin1_general_ci", MYF(0));
+    cset = ci->cset;
     coll = &mock_collation;
     mbmaxlen = 1;
     pad_attribute = PAD_SPACE;
